using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Drawing;
using System.Runtime.InteropServices;
using Sdgt;

namespace Sdgt
{
	
	/// <summary>The different types of System Mouse Emulation Modes</summary>
	/// <remarks><para>There may be cases where you actually want the system mouse to follow a particular mouse
	/// or you may want to park the system mouse in a particular location so to prevent problems which clicking 
	/// in a location outside the mouse area.  This enumeration provides that functionality.</para></remarks>
	[Description("The different types of System Mouse Emulation Modes")]
	public enum EmulateSystemMouseModes
	{
		/// <summary>Park the system mouse at the location specified by SystemMouseParkLocation</summary>
		[Description("Park the system mouse at the location specified by SystemMouseParkLocation")]
		Park = 0,
		/// <summary>Follow the mouse specified by the MouseToFollow property</summary>
		[Description("Follow the mouse specified by the MouseToFollow property")]
		FollowMouse = 1
	}
	/// <summary>The different types of coordinates provided in each mouse event</summary>
	/// <remarks><para>A programmer may be interested in delta values rather than mouse coordinates.  This 
	/// enumeration describes the different coordinate systems offered.</para></remarks>
	[Description("The different types of coordinates provided in each mouse event")]
	public enum ScaleModes
	{
		/// <summary>All coordinates are absolute, and can be made relative to a form through the RelativeTo property</summary>
		[Description("All coordinates are absolute, and can be made relative to a form through the RelativeTo property")]
		Pixel = 0,
		/// <summary>All coordinates are relative to the last set of coordinates.  The programmer works with delta values</summary>
		[Description("All coordinates are relative to the last set of coordinates.  The programmer works with delta values")]
		PixelDelta = 1
	}

	/// <summary>This is the manager for all input events and also sends events to all sdg aware widgets</summary>
	[Description("This is the manager for all input events and also sends events to all sdg aware widgets")]
	[DefaultEvent("SdgMouseMove")]
	public class SdgManager : System.ComponentModel.Component, IDisposable
	{
		#region Sdg Manager Variables 
		
		static private int iTotalSdgManagers = 0;
		private System.ComponentModel.Container components = null;
		private Control m_RelativeControl;
		private Point m_ParkLocation = new Point(350,350);
		private int m_MouseToFollow = 0;
		private EmulateSystemMouseModes m_SystemMouseEmulationMode = EmulateSystemMouseModes.Park;
		private bool m_fStarted = false;
		
		
		#endregion 

		#region Sdg Manager Functions

		/// <summary>SDG Manager Constructor: Must be added to a container before the Toolkit becomes active.</summary>
		/// <remarks><para>This constructor does not actually start the sdg manager until it is added to a container.  </para></remarks>
		/// <example><para>C# e.g., sdgManager1 = new sdgManager(); this.components.Add(sdgManager1);</para>
		/// <para>The above code assumes that this.components is not null.</para></example>
		public SdgManager()
		{
			
			//StartSdgManager();
			//Container c = new Container();
			//c.Add(this);
			//this.components = new Container();
			//Debug.Assert(false, "You must specify a container for this component");
				
			
			InitializeComponent();
			
			
		}

		/// <summary>Gets or sets the Site interface associated with this component.  The SDG Manager starts only after it has been added to a container.</summary>
		/// <remarks><para>You need not set the ISite directly.  The ISite is automatically set when the class is added to a container (e.g., this.components.Add(sdgManager1);)</para></remarks>
		[Description("Gets or sets the Site interface associated with this component.  The SDG Manager starts only after it has been added to a container.")]
		public override ISite Site
		{
			get
			{
				return base.Site;
			}
			set
			{
				base.Site = value;
				if ((false == m_fStarted)&&(null != base.Site)&&(null == impl)&&(false == this.DesignMode))
				{
					//MessageBox.Show("Setting the ISite");
					m_fStarted = true;
					StartSdgManager();
				}
			}
		}

		/// <summary>The default constructor automatically called when the component is dragged onto the form</summary>
		public SdgManager(System.ComponentModel.IContainer container)
		{
			//StartSdgManager();
			
			//StartSdgManager();
			container.Add(this);
			//this.components = (Container) container;
			InitializeComponent();
			
			


		}

		/// <summary>Default destructor for the sdgManager class</summary>
		~ SdgManager()
		{
		
		}
		/// <summary>Function used to delete the current SDG Manager and restore the computer back to it's previous state</summary>
		public new void Dispose()
		{
			if (impl != null)
			{
				Win32.SetCursorPos(impl.mice[0].Xabs, impl.mice[0].Yabs);
				impl.SdgMouseDown -= new SdgMouseEventHandler(this.OnMouseDown);
				impl.SdgMouseUp -= new SdgMouseEventHandler(this.OnMouseUp);
				impl.SdgMouseMove -= new SdgMouseEventHandler(this.OnMouseMove);
				impl.SdgMouseWheel -= new SdgMouseEventHandler(this.OnMouseWheel);
				impl.SdgMouseAttached -= new SdgMouseEventHandler(this.OnMouseAttached);
				impl.SdgKeyDown -= new SdgKeyEventHandler(this.OnKeyDown);
				impl.SdgKeyUp -= new SdgKeyEventHandler(this.OnKeyUp);
				impl.SdgKeyPress -= new SdgKeyPressEventHandler(this.OnKeyPress);
				if (null != impl.m_RelativeControl)
				{
					impl.m_RelativeControl.Capture = false;
					impl.m_RelativeControl.Cursor = Cursors.Arrow;
				}
				for (int i=0; i<impl.mice.Count; i++)
				{
					impl.mice[i].impl.Capture = false;
					impl.mice[i].impl.Dispose();
					impl.mice[i].impl.RelativeTo = null;
					Win32.ShowCursor(true);
					//impl.mice[i].impl = null;
				}
				impl.Dispose();
				impl = null;
				iTotalSdgManagers--;
			}
			base.Dispose();
		}


		

	
	

		private void StartSdgManager()
		{
			if (this.DesignMode)
			{

			}
			else
			{
				
				iTotalSdgManagers++;
				
				impl = new InputEventManagerImpl();
				impl.SdgMouseDown += new SdgMouseEventHandler(this.OnMouseDown);
				impl.SdgMouseUp += new SdgMouseEventHandler(this.OnMouseUp);
				impl.SdgMouseMove += new SdgMouseEventHandler(this.OnMouseMove);
				impl.SdgMouseWheel += new SdgMouseEventHandler(this.OnMouseWheel);
				impl.SdgMouseAttached += new SdgMouseEventHandler(this.OnMouseAttached);
				impl.SdgKeyDown += new SdgKeyEventHandler(this.OnKeyDown);
				impl.SdgKeyUp += new SdgKeyEventHandler(this.OnKeyUp);
				impl.SdgKeyPress += new SdgKeyPressEventHandler(this.OnKeyPress);
				
			}
		}


		/// <summary>Gets or sets the mouse class specified by the index parameter</summary>
		/// <remarks><para>There are several ways to iterate through all of the mice on the system.
		/// One can use a numeric iterator such as "int i" or use a mouse iterator using the foreach parameter.</para></remarks>
		/// <example><para>C# Eg., for (int i=0; i &lt; sdgManager1.Mice.Count; ++i) { sdgManager1.Mice[i].Text = "Tricks are for kids"; }</para></example>
		/// <example><para>C# Eg., foreach (Sdgt.Mouse m in sdgManager1.Mice) { m.Text = "Tricks are for kids"; }</para></example>
		[Description("Gets or sets the mouse class specified by the index parameter")]
		[Browsable(false)]
		public Mouse.MouseCollection Mice
		{
			get
			{
				if (null != impl)
				{
					return SdgManager.impl.mice;
				} 
				else
				{
					return null;
				}

			}
			set
			{
				if ((null != impl)&&(null != value))
				{
					SdgManager.impl.mice = value;
				}
			}
		}
		
		/// <summary>Gets or sets the keyboard class specified by the index parameter</summary>
		/// <remarks><para>There are several ways to iterate through all of the keyboards on the system.
		/// One can use a numeric iterator such as "int i" or use a mouse iterator using the foreach parameter.</para></remarks>
		/// <example><para>C# Eg., for (int i=0; i &lt; sdgManager1.Keyboards.Count; ++i) { }</para></example>
		/// <example><para>C# Eg., foreach (Sdgt.Keyboard k in sdgManager1.Keyboards) { }</para></example>
		[Description("Gets or sets the keyboard class specified by the index parameter")]
		[Browsable(false)]
		public Keyboard.KeyboardCollection Keyboards
		{
			get
			{
				if (null != impl)
				{
					return SdgManager.impl.keyboards;
				}
				else
				{
					return null;
				}
			}
			set
			{
				if (null != impl && null != value)
				{
					SdgManager.impl.keyboards = value;
				}
			}
		}

		/// <summary>Gets or sets the System Mouse Emulation Mode</summary>
		/// <remarks><para>There may be cases where you actually want the system mouse to follow mouse 0
		/// or you may want to park the system mouse in a particular location so to prevent problems which clicking 
		/// in a location outside the mouse area.  This enumeration provides that functionality.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.EmulateSystemMouseMode = EmulateSystemMouseModes.FollowMouse</para></example>
		[Description("Gets or sets the System Mouse Emulation Mode")]
		public EmulateSystemMouseModes EmulateSystemMouseMode
		{
			get
			{
				if (this.DesignMode)
				{
					return m_SystemMouseEmulationMode;
				}
				else
				{
					return SdgManager.impl.systemMouseEmulationMode;
				}
			}
			set
			{
				if (this.DesignMode)
				{
					m_SystemMouseEmulationMode = value;
				}
				else
				{
					SdgManager.impl.systemMouseEmulationMode = value;
				}
			}
		}

		/// <summary>Gets or sets the mouse the system mouse will follow</summary>
		/// <remarks><para>There may be cases where you actually want the system mouse to follow a particular mouse
		/// to allow the normal interaction with Windows widgets. This property specifies which system mouse should be followed</para></remarks>
		/// <example><para>C# Eg., sdgManager1.MouseToFollow = EmulateSystemMouseModes.FollowMouse</para></example>
		[Description("Gets or sets the mouse the system mouse will follow")]
		[Browsable(false)]
		public int MouseToFollow
		{
			get
			{
				if (this.DesignMode)
				{
					return m_MouseToFollow;
				}
				else
				{
					return impl.m_MouseToFollow;
				}
				
			}
			set
			{
				if (this.DesignMode)
				{
					//m_SystemMouseEmulationMode = EmulateSystemMouseModes.FollowMouse;
					m_MouseToFollow = value;
				}
				else
				{
					Debug.Assert(value >= 0 && value < Mice.Count, "Cannot follow a mouse that does not exist");
					//EmulateSystemMouseMode = EmulateSystemMouseModes.FollowMouse;
					impl.m_MouseToFollow = value;
				}
			}
		}


		


		/// <summary>Gets or sets the location where the system mouse cursor will be parked</summary>
		/// <remarks><para>To prevent the system mouse from being seen in the application and to prevent mouse clicks
		/// from going astray and losing focus of the application it may be necessary to park the mosue in a particular 
		/// location.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.ParkSystemMouseLocation = new Point(300,300);</para></example>
		[Description("Gets or sets the location where the system mouse cursor will be parked")]
		public Point ParkSystemMouseLocation
		{
			get
			{
				if (this.DesignMode)
				{
					return m_ParkLocation;
				}
				else
				{
					return impl.m_ParkLocation;
				}
			}
			set
			{
				if (this.DesignMode)
				{
					//m_SystemMouseEmulationMode = EmulateSystemMouseModes.Park;
					m_ParkLocation = value;
				}
				else
				{
					//EmulateSystemMouseMode = EmulateSystemMouseModes.Park;
					impl.m_ParkLocation = value;
				}
			}
		}

		/// <summary>Gets or sets the Control class to which all mouse events will be relative to</summary>
		/// <remarks><para>This function makes all cursor coordinates relative to a particular part of 
		/// the screen.  This function allows a user to make their coordinates relative to a particular control.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.RelativeTo = this;</para></example>
		[Description("Gets or sets the Control class to which all mouse events will be relative to")]
		
		public Control RelativeTo
		{
			get
			{
				
				if (null != impl)
				{
					return impl.topLevelControl;
				}
				else
				{
					return this.m_RelativeControl;
				}

			}
			set
			{
				if (null != impl)
				{
					if (null != value)
					{
						for (int i=0; i< Mice.Count; i++)
						{
							Mice[i].RelativeTo = value; 
						}
						Point p = value.PointToScreen(new Point(0,0));
						
						
						impl.m_ParkLocation = new Point(p.X + (impl.TopLevelControl.Width /2), p.Y + (impl.TopLevelControl.Height / 2));
					}
					impl.topLevelControl = value;
					
				}
				else
				{
					this.m_RelativeControl = value;
				}
				
			}
		}

	
		/// <summary>Adds a widget that will recieve all SDG Mouse and Keyboard events (e.g., the SDG Form)</summary>
		/// <param name="widget">The widget that will recieve SDG Mouse and Keyboard events</param>
		/// <param name="RelativeTo">The control that coorindates should be relative to</param>
		[Description("Adds a widget that will recieve all SDG Mouse and Keyboard events (e.g., the SDG Form)")]
		public void AttachEventSinkWidget(ISdgKeyboardWidget widget, Control RelativeTo)
		{
			impl.m_EventSinkWidgets.Add((object) widget);
			impl.m_EventSinkRelativeTo.Add((object) RelativeTo);
		}

		/// <summary>Adds a widget that will recieve all SDG Mouse and Keyboard events (e.g., the SDG Form)</summary>
		/// <param name="widget">The widget that will recieve SDG Mouse and Keyboard events</param>
		[Description("Adds a widget that will recieve all SDG Mouse and Keyboard events (e.g., the SDG Form)")]
		public void AttachEventSinkWidget(ISdgKeyboardWidget widget)
		{
			impl.m_EventSinkWidgets.Add((object) widget);
			impl.m_EventSinkRelativeTo.Add(null);
		}

		#endregion


		#region Input Manager Class 

		internal class InputEventManagerImpl : Form
		{
			internal EmulateSystemMouseModes systemMouseEmulationMode = EmulateSystemMouseModes.Park;
			internal Point m_ParkLocation = new Point(350, 350);
			internal int m_MouseToFollow = 0;
			internal Mouse.MouseCollection mice; 
			internal Keyboard.KeyboardCollection keyboards; 
			internal ScaleModes m_ScaleMode = ScaleModes.Pixel;
			internal WidgetCollection Widgets;
			internal ArrayList allControls = null;
			internal ControlEventHandler controlEventHandler;
			internal Control m_RelativeControl = null; 
			internal ArrayList m_RawInputList;
			internal int m_RawInputListSize = 1;
			internal System.Threading.Timer m_PerformanceTimer;
			internal int m_RawInputListMaxSize = 7;
			internal int m_MoveEventCount;
			internal int m_RawInputListUpperThreshold = 6; //max mouse move events per 100ms
			internal int m_RawInputListLowerThreshold = 4; //min mouse event per 100ms
			internal float m_SystemUseageThreshold = 75.0f;
			internal PerformanceCounter m_performancecounter;

			//Variables for Widgets
			internal ArrayList m_EventSinkWidgets;
			internal ArrayList m_EventSinkRelativeTo;

			public InputEventManagerImpl()
			{
				
				m_EventSinkWidgets = new ArrayList();
				m_EventSinkRelativeTo = new ArrayList();

				m_RawInputListSize = 0;
				m_RawInputList  = new ArrayList();
				m_MoveEventCount = 0;
				
				m_performancecounter = new PerformanceCounter("Processor","% Processor Time", "_Total", true);
				
				//m_RelativeControl = (Form) System.Windows.Forms.Form.ActiveForm.;
				int iNumDevices =0 ;
				Win32.GetRawInputDeviceList(System.IntPtr.Zero, ref iNumDevices, Marshal.SizeOf(typeof(Win32.RAWINPUTDEVICELIST)));
				
				Win32.RAWINPUTDEVICELIST[] pridl = new Win32.RAWINPUTDEVICELIST[Constants.MAXMICE];
				int cbSize = iNumDevices * Marshal.SizeOf(typeof(Win32.RAWINPUTDEVICELIST));
				int iResult = Win32.GetRawInputDeviceList(pridl, ref iNumDevices, Marshal.SizeOf(typeof(Win32.RAWINPUTDEVICELIST)));
				Debug.Assert(iResult > 0, "Get Raw Input Device List Failed");
				
				ArrayList MouseArray = new ArrayList();
				ArrayList KeyboardArray = new ArrayList();

				for (int i=0; i<iNumDevices; i++)
				{
					if (((int) Win32.RawInputDeviceType.RIM_TYPEMOUSE == pridl[i].dwType)&&(System.IntPtr.Zero != pridl[i].hDevice))
					{
						Win32.RID_DEVICE_INFO rdi;
						rdi = new Win32.RID_DEVICE_INFO();
						int junk = Marshal.SizeOf(rdi);
						rdi.cbSize = Marshal.SizeOf(rdi);
						iResult = Win32.GetRawInputDeviceInfo(pridl[i].hDevice,(int) Win32.RawInputDeviceInfoCommand.RIDI_DEVICEINFO,ref rdi, ref junk);
						Debug.Assert(iResult != -1, "Get Raw Input Device Info Failed");
						if (!(rdi.mouse.dwNumberOfButtons == 2 && rdi.mouse.dwSampleRate == 60 && rdi.mouse.dwId == 2)) 
						{
							Mouse m = new Mouse(pridl[i].hDevice,500 + MouseArray.Count * 10, 500);
							if (m_RelativeControl != null)
							{
								m.RelativeTo = m_RelativeControl;
							}

							MouseArray.Add(m);		
							m_RawInputList.Add(new ArrayList());
						}

					} 
					else if ((int) Win32.RawInputDeviceType.RIM_TYPEKEYBOARD == pridl[i].dwType && IntPtr.Zero != pridl[i].hDevice) 
					{
						Win32.RID_DEVICE_INFO rdi;
						rdi = new Win32.RID_DEVICE_INFO();
						int junk = Marshal.SizeOf(rdi);
						rdi.cbSize = Marshal.SizeOf(rdi);
						iResult = Win32.GetRawInputDeviceInfo(pridl[i].hDevice,(int) Win32.RawInputDeviceInfoCommand.RIDI_DEVICEINFO,ref rdi, ref junk);
						Debug.Assert(iResult != -1, "Get Raw Input Device Info Failed");
						if (!(rdi.keyboard.dwKeyboardType == 81 && rdi.keyboard.dwNumberOfKeysTotal == 101)) 
						{	
							KeyboardArray.Add(new Keyboard(pridl[i].hDevice));	
						}

						
					}

				}
				mice = new Mouse.MouseCollection(MouseArray);
				keyboards = new Keyboard.KeyboardCollection(KeyboardArray);
				if (iTotalSdgManagers > 1)
				{ // if this is not the primary manager it is not necessary to draw multiple mice
					foreach (Mouse m in mice)
					{
						m.Visible = false;
					}
				}
				//Fire all of the apropriate attached events
				for(int i=0; i < mice.Count; ++i)
				{
					SdgMouseEventArgs e = new SdgMouseEventArgs(i,MouseButtons.None,0,mice[i].X,mice[i].Y,0);
					OnSdgMouseAttached(this, e);
				}
				for(int i=0; i < keyboards.Count; ++i)
				{
					SdgKeyPressEventArgs e = new SdgKeyPressEventArgs();
					e.ID = i;
					e.KeyChar = (char) 0;
					OnSdgKeyAttached(this, e);
					if (i < mice.Count)
					{
						keyboards[i].Mouse = i;
					}
				}


				Text = "Control Class";
				BackColor = Color.AliceBlue;
				TransparencyKey = Color.AliceBlue;
				
				ControlBox = false;
				TopMost = false;
				Capture = true;
				CausesValidation = false;
				Cursor = Cursors.No;
				FormBorderStyle = FormBorderStyle.None;
				ShowInTaskbar = false;
				
				Win32.ShowCursor(false);
				Win32.SetCursorPos(300,300);

				this.controlEventHandler = new ControlEventHandler(this.topLevelControl_ControlEventHandler);
				
				Widgets = new WidgetCollection(new ArrayList());
				this.Disposed += new EventHandler(killSDGControl);
				
				if (null == m_PerformanceTimer)
				{
					
					m_PerformanceTimer = new System.Threading.Timer(new System.Threading.TimerCallback(PerformanceTimer_Tick),null,100,100);
					
				}

			}
			protected void killSDGControl(object sender, EventArgs e)
			{
				Capture = false;
				Cursor = Cursors.Arrow;
				Win32.ShowCursor(true);
				m_PerformanceTimer.Dispose();
				
				
			}


			protected void PerformanceTimer_Tick(object state)
			{
				
				if ((m_RawInputListUpperThreshold < m_MoveEventCount)&&(m_RawInputListMaxSize > m_RawInputListSize))
				{
					m_RawInputListSize++;
				}
				else if ((m_RawInputListLowerThreshold >  m_MoveEventCount)&&(m_RawInputListSize > 0))
				{
					m_RawInputListSize--;
				}
				m_MoveEventCount = 0;


			}

			protected unsafe override void WndProc(ref Message m)
			{
				switch((Win32.WindowMessage) m.Msg)
				{
					case Win32.WindowMessage.WM_INPUT:
					{
							

						Win32.RAWINPUT ri = new Win32.RAWINPUT();
						int cbri = Marshal.SizeOf(ri);
						Win32.GetRawInputData((Win32.RAWINPUT*) ((void*) m.LParam), (int) Win32.GetRawInputDataCommand.RID_INPUT, ref ri, ref cbri, Marshal.SizeOf(ri.header));
						if (ri.header.dwType == (int) Win32.RawInputDeviceType.RIM_TYPEMOUSE)
						{
							
							bool foundMouse = false;
							

							for (int i = 0; i < mice.Count; ++i)
							{
								if (new IntPtr(ri.header.hDevice) == mice[i].impl.m_hDevice) 
								{
									ArrayList a = (ArrayList) m_RawInputList[i];
									if (a.Count < m_RawInputListSize && 0 == ri.mouse.mouseUnion.usButtonData && 0 == ri.mouse.mouseUnion.usButtonFlags)
									{
										a.Add(ri);
									}
									else
									{
										m_MoveEventCount++;
									
										for (int j=0; j< a.Count; j++)
										{
											Win32.RAWINPUT ri2 = (Win32.RAWINPUT) a[j];
											ri.mouse.lLastX += ri2.mouse.lLastX;
											ri.mouse.lLastY += ri2.mouse.lLastY;
										}
										m_RawInputList[i] = new ArrayList();

										if (systemMouseEmulationMode == EmulateSystemMouseModes.Park)
										{
											mice[i].impl.ChangeXY(ri.mouse.lLastX, ri.mouse.lLastY, m_ParkLocation.X, m_ParkLocation.Y);
										}
										else if (systemMouseEmulationMode == EmulateSystemMouseModes.FollowMouse)
										{
											mice[i].impl.ChangeXY(ri.mouse.lLastX, ri.mouse.lLastY, mice[m_MouseToFollow].Xabs, mice[m_MouseToFollow].Yabs);
										}

										SdgMouseEventArgs smea;
										SdgMouseEventArgs smeaAllButtons;
										
									
										short delta = (short) ri.mouse.ulRawButtons;
										if (ScaleModes.Pixel == m_ScaleMode) 
										{
											smea = new SdgMouseEventArgs(i,(MouseButtons) RawToMouseButton(ri.mouse.mouseUnion.usButtonData), ri.mouse.mouseUnion.usButtonFlags, mice[i].X, mice[i].Y, delta);
											smeaAllButtons = new SdgMouseEventArgs(i, mice[i].Button, ri.mouse.mouseUnion.usButtonFlags, mice[i].X, mice[i].Y, delta);
										} 
										else 
										{ //sdgPixelDelta
											smea = new SdgMouseEventArgs(i,(MouseButtons) RawToMouseButton(ri.mouse.mouseUnion.usButtonData), ri.mouse.mouseUnion.usButtonFlags, ri.mouse.lLastX, ri.mouse.lLastY, delta);
											smeaAllButtons = new SdgMouseEventArgs(i, mice[i].Button, ri.mouse.mouseUnion.usButtonFlags, ri.mouse.lLastX, ri.mouse.lLastY, delta);
										}
										object mySender = this;
										//figure out if it is over a particular widget
										if (null != allControls)
										{
											for (int d = allControls.Count -1; d >= 0; d--)
											{
												Control myControl = (Control) allControls[d];
												if (myControl.Bounds.Contains(mice[i].X, mice[i].Y))
												{
													mySender = allControls[d];
													break;
												}
											}
										}

										OnSdgMouseMove(mySender, smeaAllButtons);
										mice[i].OnMouseMove(smeaAllButtons);

										bool fMouseDown = false;
										bool fMouseUp = false;
										

										if ((short) Win32.RI_MOUSE.RI_MOUSE_WHEEL == ri.mouse.mouseUnion.usButtonData) 
										{
											OnSdgMouseWheel(mySender, smea);
											mice[i].OnMouseWheel(smea);
											//fMouseWheel = true;
										}

										switch (ri.mouse.mouseUnion.usButtonData)
										{
											case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_UP:
											case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_UP:
											case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_UP:
											case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_UP:
											case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_UP:	
											
												OnSdgMouseUp(mySender, smea);
												fMouseUp = true;
											
												mice[i].OnMouseUp(smea);
												mice[i].setButtonState(SmeaToState(ri.mouse.mouseUnion.usButtonData), false);
											
												if (mice[i].Focus != null && mice[i].MouseButtonsOnWidget > 0)
												{
													Control mouseControl = (Control) mice[i].Focus;
													SdgMouseEventArgs smeaWidget = new SdgMouseEventArgs(smea.ID, smea.Button, smea.X - mouseControl.Location.X, smea.Y - mouseControl.Location.Y, smea.Delta);
													mice[i].Focus.OnSdgMouseUp(smeaWidget);
													mice[i].MouseButtonsOnWidget--;
												}
												break;
											case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_DOWN:
											case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_DOWN:
											case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_DOWN:
											case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_DOWN:
											case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_DOWN:	
												OnSdgMouseDown(mySender, smea);
												mice[i].OnMouseDown(smea);
												mice[i].setButtonState(SmeaToState(ri.mouse.mouseUnion.usButtonData), true);
												fMouseDown = true;
								
												if(mice[i].Focus != null && mice[i].MouseButtonsOnWidget > 0)
												{											
													Control mouseControl = (Control) mice[i].Focus;
													SdgMouseEventArgs smeaWidget = new SdgMouseEventArgs(smea.ID, smea.Button, smea.X - mouseControl.Location.X, smea.Y - mouseControl.Location.Y, smea.Delta);
													mice[i].Focus.OnSdgMouseDown(smeaWidget);
													mice[i].MouseButtonsOnWidget++;
												}
											
												break;
										}


										for (int w=0; w < Widgets.Count; ++w)
										{
											if (Widgets[w] is ISdgMouseWidget)
											{
												ISdgMouseWidget mouseWidget = (ISdgMouseWidget) Widgets[w];
												Control mouseControl = (Control) Widgets[w];
												if (mouseControl.Bounds.Contains(new Point(smea.X,smea.Y)))
												{
													SdgMouseEventArgs smeaWidget = new SdgMouseEventArgs(smea.ID, smea.Button, smea.X - mouseControl.Location.X, smea.Y - mouseControl.Location.Y, smea.Delta);
													mouseWidget.OnSdgMouseMove(smeaWidget);
													if ((true == fMouseDown)&&(mice[i].MouseButtonsOnWidget <= 0))
													{
														mouseWidget.OnSdgMouseDown(smeaWidget);
														
														mice[i].Focus = mouseWidget;
														mice[i].MouseButtonsOnWidget++;
													}
													if ((true == fMouseUp)&&(mice[i].MouseButtonsOnWidget > 0))
													{
														mouseWidget.OnSdgMouseClick(smeaWidget);
														
													}
											
												
												}
											}
										}
										if ((int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_DOWN ==  ri.mouse.mouseUnion.usButtonData)
										{
											if (null != allControls)
											{
												foreach (Control c in allControls)
												{
													if (c.Bounds.Contains(smea.X,smea.Y))
													{
														mice[i].ControlFocus = c;	
													}
												}
											}
										}

									} //else (RawInputList processed);

									foundMouse = true;
									break;								

								}
								foundMouse = false;
							}
							if (false == foundMouse)
							{ //someone added a new mouse!
								Win32.RID_DEVICE_INFO rdi = new Win32.RID_DEVICE_INFO();
								m_RawInputList.Add(new ArrayList());
								int junk = Marshal.SizeOf(rdi);
								rdi.cbSize = Marshal.SizeOf(rdi);
								Win32.GetRawInputDeviceInfo(new IntPtr(ri.header.hDevice), (int) Win32.RawInputDeviceInfoCommand.RIDI_DEVICEINFO, ref rdi, ref junk);
								int MouseNumber = mice.Count;
								if (!(rdi.mouse.dwNumberOfButtons == 2 && rdi.mouse.dwSampleRate == 60 && rdi.mouse.dwId == 2)) 
								{
									mice[MouseNumber] = new Mouse(new IntPtr(ri.header.hDevice), 500 + mice.Count * 10, 500);
									if (null != m_RelativeControl) 
									{
										mice[MouseNumber].RelativeTo = m_RelativeControl;
									}

									SdgMouseEventArgs smea;
									int delta = ri.mouse.mouseUnion.usButtonData;

									if (ScaleModes.Pixel == m_ScaleMode) 
									{
										smea = new SdgMouseEventArgs(MouseNumber,mice[MouseNumber].Button, ri.mouse.mouseUnion.usButtonFlags, mice[MouseNumber].X, mice[MouseNumber].Y, delta);
									} 
									else 
									{ //sdgPixelDelta
										smea = new SdgMouseEventArgs(MouseNumber,mice[MouseNumber].Button, ri.mouse.mouseUnion.usButtonFlags, ri.mouse.lLastX, ri.mouse.lLastY, delta);
									}
									OnSdgMouseAttached(this, smea );
									
									
								}

								
							}
						}
						else if (ri.header.dwType == (int) Win32.RawInputDeviceType.RIM_TYPEKEYBOARD) 
						{
							bool fFoundKeyboard = false;
							for (int i=0; i < keyboards.Count; ++i)
							{

								if (keyboards[i].m_hDevice.ToInt32() == ri.header.hDevice)
								{ //we found the keyboard to work with
									fFoundKeyboard = true;
									SdgKeyEventArgs skea = new SdgKeyEventArgs();
									SdgKeyPressEventArgs skpea = new SdgKeyPressEventArgs();

									keyboards[i].handleKeyboardInput((int) ri.keyboard.VKey, ri.keyboard.Message, skea, i);
									skpea.ID = i;
									skpea.KeyChar = (char) skea.KeyValue;
									

									switch (ri.keyboard.Message)
									{
									case (int) Win32.WindowMessage.WM_KEYDOWN:
									case (int) Win32.WindowMessage.WM_SYSKEYDOWN:
										OnSdgKeyDown(this, skea);
										OnSdgKeyPress(this, skpea);
										keyboards[i].OnKeyDown(skea);
										keyboards[i].OnKeyPress(skpea);
										if (keyboards[i].Mouse > -1 && mice[keyboards[i].Mouse].Focus != null && mice[keyboards[i].Mouse].Focus is ISdgKeyboardWidget)
										{
											ISdgKeyboardWidget keyWidget = (ISdgKeyboardWidget) mice[keyboards[i].Mouse].Focus;
											keyWidget.OnSdgKeyDown(skea);
											keyWidget.OnSdgKeyPress(skpea);											
										}
										break;	
									case (int) Win32.WindowMessage.WM_KEYUP:
									case (int) Win32.WindowMessage.WM_SYSKEYUP:
										OnSdgKeyUp(this, skea);
										keyboards[i].OnKeyUp(skea);
										if (keyboards[i].Mouse > -1 && mice[keyboards[i].Mouse].Focus != null && mice[keyboards[i].Mouse].Focus is ISdgKeyboardWidget)
										{
											ISdgKeyboardWidget keyWidget = (ISdgKeyboardWidget) mice[keyboards[i].Mouse].Focus;
											keyWidget.OnSdgKeyUp(skea);					
										}
										break;
									}
									break;
								}
							}
							if (false == fFoundKeyboard)
							{
								int KeyboardNumber = keyboards.Count;
								Win32.RID_DEVICE_INFO rdi;
								rdi = new Win32.RID_DEVICE_INFO();
								int junk = Marshal.SizeOf(rdi);
								rdi.cbSize = Marshal.SizeOf(rdi);
								int iResult = Win32.GetRawInputDeviceInfo(new IntPtr(ri.header.hDevice),(int) Win32.RawInputDeviceInfoCommand.RIDI_DEVICEINFO,ref rdi, ref junk);
								Debug.Assert(iResult != -1, "Get Raw Input Device Info Failed");
								if (!(rdi.keyboard.dwKeyboardType == 81 && rdi.keyboard.dwNumberOfKeysTotal == 101)) 
								{
									
									keyboards[KeyboardNumber] = new Keyboard(new IntPtr(ri.header.hDevice));					
									SdgKeyPressEventArgs e = new SdgKeyPressEventArgs();
									e.ID = KeyboardNumber;
									e.KeyChar = (char) 0;
									OnSdgKeyAttached(this, e);
									
								}
								

							}
						}
						
						
					}
						break;
					case Win32.WindowMessage.WM_CREATE:
					{
						Win32.RAWINPUTDEVICE[] rid = new Win32.RAWINPUTDEVICE[2];
			
						rid[0].usUsagePage = 1;
						rid[0].usUsage = 2;
						rid[0].dwFlags = 0;
						rid[0].hwndTarget = m.HWnd;

						rid[1].usUsagePage = 1;
						rid[1].usUsage = 6;
						rid[1].dwFlags = 0;
						rid[1].hwndTarget = m.HWnd;

						bool fResult = Win32.RegisterRawInputDevices(rid,2,Marshal.SizeOf(rid[0]));
						Debug.Assert(fResult == true, "Failed to Register Raw Input Devices");

					}	
						break;
					default:
						base.WndProc(ref m);
						break;
				}
			}
			public int SmeaToState(int s)
			{
				switch (s)
				{
					case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_UP:
						return 0;
					case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_UP:
						return 1;
					case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_UP:
						return 2;
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_DOWN: 
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_UP: 
						return 3;
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_DOWN: 
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_UP: 
						return 4;
					default:
						return Constants.MAXMOUSEBUTTONS;
				}
			}

			public MouseButtons RawToMouseButton(int s)
			{
				switch (s)
				{
					case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_UP:
						return MouseButtons.Left;
					case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_UP:
						return MouseButtons.Right;
					case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_DOWN:
					case (int) Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_UP:
						return MouseButtons.Middle;
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_DOWN: 
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_UP: 
						return MouseButtons.XButton1;
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_DOWN: 
					case (int) Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_UP: 
						return MouseButtons.XButton2;
					case (int) Win32.RI_MOUSE.RI_MOUSE_WHEEL:
						return MouseButtons.Middle;
					default:
						return MouseButtons.None;
				}
			}

			public MouseButtons StateToSmea(int state)
			{
				switch (state)
				{
					case 0:
						return MouseButtons.Left;
					case 1:
						return MouseButtons.Right;
					case 2:
						return MouseButtons.Middle;
					case 3: 
						return MouseButtons.XButton1;
					case 4:
						return MouseButtons.XButton2;
					default:
						return MouseButtons.None;
				}
			}

			

	
			public Control topLevelControl
			{
				get
				{
					return this.m_RelativeControl;
				}
				set
				{
					if(null != this.m_RelativeControl)
					{
						this.m_RelativeControl.ControlAdded -= this.controlEventHandler;
						this.m_RelativeControl.ControlRemoved -= this.controlEventHandler;
						
					}
					this.m_RelativeControl = value;
					if(null != this.m_RelativeControl)
					{
						this.m_RelativeControl.ControlAdded += this.controlEventHandler;
						this.m_RelativeControl.ControlRemoved += this.controlEventHandler;
						
					}
					this.topLevelControl_ControlEventHandler(this.m_RelativeControl, null);
				}
			}

			private void topLevelControl_ControlEventHandler(object sender, ControlEventArgs e)
			{
				ArrayList al = new ArrayList();
				ArrayList all = new ArrayList();
				WalkControlHierarchy(all, al, this.m_RelativeControl);
				Widgets = new WidgetCollection(al);
				allControls = all;
				
			}

			

			private static void WalkSdgWidgetHierarchy(ArrayList al, Control p)
			{
				if (null != p)
				{
					if (p.Name != "")
					{
						Console.WriteLine("Inner User Controled Detected " + p.Name);
						al.Add(p);
					}
					foreach(Control c in p.Controls)
					{
						WalkSdgWidgetHierarchy(al, c);
					}
				}
			}

			private static void WalkControlHierarchy(ArrayList all, ArrayList al, Control p)
			{
				if(null != p)
				{
					Console.WriteLine("Control Name " + p.ToString());
					

					if(p is ISdgMouseWidget || p is ISdgKeyboardWidget)
					{
						
						
//						foreach (Control c in p.Controls)
//						{
//							WalkSdgWidgetHierarchy(al, c);
//						}
						//Console.WriteLine("SDG Control " + p.Name);
						al.Add(p);
						all.Add(p);
					}
					else
					{
						//Console.WriteLine("Control " + p.Name);
						if (p.Name != "")
							all.Add(p);
						foreach(Control c in p.Controls)
						{
							WalkControlHierarchy(all, al, c);
						}
					}
					
					
					//Console.WriteLine("Normal Control " + p.Name);
					
					
				}
			}

			

			#region InputManager Events

			public event SdgMouseEventHandler SdgMouseMove;
			public event SdgMouseEventHandler SdgMouseDown;
			public event SdgMouseEventHandler SdgMouseUp;
			public event SdgMouseEventHandler SdgMouseWheel;
			public event SdgMouseEventHandler SdgMouseAttached;
			public event SdgKeyEventHandler SdgKeyDown;
			public event SdgKeyEventHandler SdgKeyUp;
			public event SdgKeyPressEventHandler SdgKeyPress;
			public event SdgKeyPressEventHandler SdgKeyAttached;


			protected virtual void OnSdgMouseMove(object sender, SdgMouseEventArgs e)
			{
				if (SdgMouseMove != null) 
					SdgMouseMove(sender, e);
				
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];
					if (null != m_EventSinkRelativeTo[i])
					{
						Control c = (Control) m_EventSinkRelativeTo[i];
						widget.OnSdgMouseMove(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs - c.Location.X, this.mice[e.ID].Yabs - c.Location.Y, e.Delta));
					}
					else
						widget.OnSdgMouseMove(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs, this.mice[e.ID].Yabs, e.Delta));
				}
			}
		
			protected virtual void OnSdgMouseDown(object sender, SdgMouseEventArgs e)
			{
				if (SdgMouseDown != null)
					SdgMouseDown(sender, e);
				
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];
					if (null != m_EventSinkRelativeTo[i])
					{
						Control c = (Control) m_EventSinkRelativeTo[i];
						widget.OnSdgMouseDown(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs - c.Location.X, this.mice[e.ID].Yabs - c.Location.Y, e.Delta));
					}
					else
						widget.OnSdgMouseDown(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs, this.mice[e.ID].Yabs, e.Delta));
				}
			}
		
			protected virtual void OnSdgMouseUp(object sender, SdgMouseEventArgs e)
			{
				if (SdgMouseUp != null)
					SdgMouseUp(sender, e);
				
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];
					if (null != m_EventSinkRelativeTo[i]){
						Control c = (Control) m_EventSinkRelativeTo[i];
						widget.OnSdgMouseUp(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs - c.Location.X, this.mice[e.ID].Yabs - c.Location.Y, e.Delta));
					}
					else
						widget.OnSdgMouseUp(new SdgMouseEventArgs(e.ID, e.Button, this.mice[e.ID].Xabs, this.mice[e.ID].Yabs, e.Delta));
				}
			}
		
			protected virtual void OnSdgMouseWheel(object sender, SdgMouseEventArgs e)
			{
				if (SdgMouseWheel != null)
					SdgMouseWheel(sender, e);

			}
		
			protected virtual void OnSdgMouseAttached(object sender, SdgMouseEventArgs e)
			{
				if (SdgMouseAttached != null)
					SdgMouseAttached(sender, e);
			}
		
			protected virtual void OnSdgKeyDown(object sender, SdgKeyEventArgs e)
			{
				if (SdgKeyDown != null)
					SdgKeyDown(sender, e);
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];	
					widget.OnSdgKeyDown(e);
				}
			}
		
			protected virtual void OnSdgKeyUp(object sender, SdgKeyEventArgs e)
			{
				if (SdgKeyUp != null)
					SdgKeyUp(sender, e);
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];	
					widget.OnSdgKeyUp(e);
				}
			}
		
			protected virtual void OnSdgKeyPress(object sender, SdgKeyPressEventArgs e)
			{
				if (SdgKeyPress != null)
					SdgKeyPress(sender, e);
				for (int i=0; i < m_EventSinkWidgets.Count; ++i)
				{
					ISdgKeyboardWidget widget = (ISdgKeyboardWidget) m_EventSinkWidgets[i];	
					widget.OnSdgKeyPress(e);
				}
			}

			protected virtual void OnSdgKeyAttached(object sender, SdgKeyPressEventArgs e)
			{
				if (SdgKeyAttached != null)
					SdgKeyAttached(sender, e);
				
			}

		#endregion 



			public class WidgetCollection : ReadOnlyCollectionBase
			{
				
				public WidgetCollection(ArrayList value)
				{
					foreach (Object o in value)
					{
						InnerList.Add(o);
					}
				}

				
				public object this[int index]
				{
					get
					{
						return (object) InnerList[index];
					}
					set
					{
						if (index == InnerList.Count)
						{
							InnerList.Add(value);
						}
						else 
						{
							InnerList[index] = value;
						}
					
					}
				}
			}


		}
		
		private static InputEventManagerImpl impl; 
		#endregion

		#region Sdg Toolkit Events

		/// <summary>Event fired when any mouse event occurs</summary>
		[Description("Event fired when any mouse event occurs")]
		public event SdgMouseEventHandler MouseMove;

		/// <summary>Event fired when any mouse button has been pressed</summary>
		[Description("Event fired when any mouse button has been pressed")]
		public event SdgMouseEventHandler MouseDown;

		/// <summary>Event fired when any mouse button has been released</summary>
		[Description("Event fired when any mouse button has been released")]
		public event SdgMouseEventHandler MouseUp;

		/// <summary>Event fired when the mouse wheel has been moved</summary>
		[Description("Event fired when the mouse wheel has been moved")]
		public event SdgMouseEventHandler MouseWheel;

		/// <summary>Event fired when a new mouse has been detected</summary>
		[Description("Event fired when a new mouse has been detected")]
		public event SdgMouseEventHandler MouseAttached;

		/// <summary>Event fired when a key is pressed</summary>
		[Description("Event fired when a key is pressed")]
		public event SdgKeyEventHandler KeyDown;

		/// <summary>Event fired when a key is released</summary>
		[Description("Event fired when a key is released")]
		public event SdgKeyEventHandler KeyUp;

		/// <summary>Event fired when a key is pressed</summary>
		[Description("Event fired when a key is pressed")]
		public event SdgKeyPressEventHandler KeyPress;

		/// <summary>Event fired when a new keyboard has been detected</summary>
		[Description("Event fired when a new keyboard has been detected")]
		public event SdgKeyPressEventHandler KeyAttatched;

		internal void OnMouseMove(object sender, SdgMouseEventArgs e)
		{
			if (MouseMove != null) 
				MouseMove(sender, e);
		}
		
		internal void OnMouseDown(object sender, SdgMouseEventArgs e)
		{
			if (MouseDown != null)
				MouseDown(sender, e);

		}
		
		internal void OnMouseUp(object sender, SdgMouseEventArgs e)
		{
			if (MouseUp != null)
				MouseUp(sender, e);
		}
		
		internal void OnMouseWheel(object sender, SdgMouseEventArgs e)
		{
			if (MouseWheel != null)
				MouseWheel(sender, e);
		}
		
		internal void OnMouseAttached(object sender, SdgMouseEventArgs e)
		{
			if (MouseAttached != null)
				MouseAttached(sender, e);
		}
		
		internal void OnKeyDown(object sender, SdgKeyEventArgs e)
		{
			if (KeyDown != null)
				KeyDown(sender, e);
		}
		
		internal void OnKeyUp(object sender, SdgKeyEventArgs e)
		{
			if (KeyUp != null)
				KeyUp(sender, e);
		}
		
		internal void OnKeyPress(object sender, SdgKeyPressEventArgs e)
		{
			if (KeyPress != null)
				KeyPress(sender, e);
		}

		internal void OnKeyAttached(object sender, SdgKeyPressEventArgs e)
		{
			if (KeyAttatched != null)
			{
				KeyAttatched(sender, e);
			}
		}

		#endregion 

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		///</summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}
