using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;


namespace Sdgt
{
	#region Sdg Keyboard Class	

	/// <summary>A Single Display Groupware Keyboard</summary>
	/// <remarks><para>This class provides little functionality other than a set of events specific
	/// to this keyboard and storage of all the individual key states.</para></remarks>
	[Description("A Single Display Groupware Keyboard")]
	public class Keyboard
	{

		internal IntPtr m_hDevice;
		internal bool[] m_KeyStates; //the state of a keyboard in vkeys
		internal bool m_Shift = false;
		internal bool m_Control = false;
		internal bool m_Alt = false;
		internal int m_Mouse = -1;

		/// <summary>The Default Keyboard Constructor</summary>
		public Keyboard (IntPtr value)
		{
			m_hDevice = value;

			m_KeyStates = new bool[Constants.MAXKEYS + 1];
			for (int i=0; i< Constants.MAXKEYS; ++i)
			{
				m_KeyStates[i] = false;

			}
			//figure out how to add num lock states and caps lock states and 
			
			m_KeyStates[(int) Win32.VirtKey.VK_NUMLOCK] = (0 < Win32.GetKeyState((int) Win32.VirtKey.VK_NUMLOCK))?true:false;
			m_KeyStates[(int) Win32.VirtKey.VK_CAPITAL] = (0 < Win32.GetKeyState((int) Win32.VirtKey.VK_CAPITAL))?true:false;
			m_KeyStates[(int) Win32.VirtKey.VK_SCROLL] = (0 < Win32.GetKeyState((int) Win32.VirtKey.VK_SCROLL))?true:false;
			

		}

		/// <summary>Gets or sets the current mouse whose focus all keyboard events will be mapped to</summary>
		/// <remarks><para>Single Display Groupare often breaks many rules in traditional single user applications.
		/// The problem is in single display groupware you may have a one mouse and two keyboard situation.  In this case it may
		/// be apropriate to map two keyboards to one mouse focus.  This utility allows the programmer to map the various
		/// keyboards on the system to whichever mouse they desire.</para></remarks>
		/// <example><para>C# Eg., SdgManager1.Keyboards[0].Mouse = 0; SdgManager1.Keyboards[1].Mouse = 0;</para></example>
		[Description("Gets or sets the current mouse whose focus all keyboard events will be mapped to")]
		public int Mouse
		{
			get
			{
				return m_Mouse;
			}
			set
			{
				if (value >= 0)
					m_Mouse = value;
			}
		}

		internal void handleKeyboardInput(int vKey, int Message, SdgKeyEventArgs e, int KeyboardID)
		{
			setKeyState(vKey, Message);
			e.Alt = m_Alt;
			e.Control = m_Control;
			e.Shift = m_Shift;
			e.ID = KeyboardID;
			e.KeyCode = (Keys) vKey;
			e.KeyData = VKeyToKeys(vKey);
			e.KeyValue = (int) VKeyToAscii(vKey);
			Keys k = Keys.None;
			if (true == m_Shift) 
			{
				k = (Keys) Keys.Shift;
			}
			if (true == m_Control) 
			{
				k = (Keys) (k | Keys.Control);
			}
			if (true == m_Alt) 
			{
				k = (Keys) (k | Keys.Alt);
			}
			e.Modifiers = k;
		}


		internal void setKeyState(int vKey, int Message) 
		{
			if ((vKey >= 0)&&(vKey <= Constants.MAXKEYS))
			{
				if (vKey == (int) Win32.VirtKey.VK_CAPITAL || vKey == (int) Win32.VirtKey.VK_NUMLOCK || vKey == (int) Win32.VirtKey.VK_SCROLL)
				{	
					if ((int) Win32.WindowMessage.WM_KEYUP == Message) 
					{
						if (m_KeyStates[vKey] == false) 
						{ //toggle state keys
							m_KeyStates[vKey] = true;
						} 
						else 
						{
							m_KeyStates[vKey] = false;
						}
					}
				} 
				else 
				{ //regular key
					if ((int) Win32.WindowMessage.WM_KEYDOWN == Message)	
					{
						m_KeyStates[vKey] = true;
					} 
					else if ((int) Win32.WindowMessage.WM_KEYUP == Message) 
					{
						m_KeyStates[vKey] = false;
					}
				}
				//set the three modifiers
				m_Shift = (true == m_KeyStates[(int) Win32.VirtKey.VK_SHIFT] || true == m_KeyStates[(int) Win32.VirtKey.VK_LSHIFT] || true == m_KeyStates[(int) Win32.VirtKey.VK_RSHIFT] || true == m_KeyStates[(int) Win32.VirtKey.VK_CAPITAL])? true:false;
				m_Alt = (true == m_KeyStates[(int) Win32.VirtKey.VK_MENU] || true == m_KeyStates[(int) Win32.VirtKey.VK_LMENU] || true == m_KeyStates[(int) Win32.VirtKey.VK_RMENU])? true:false;
				m_Control = (true == m_KeyStates[(int) Win32.VirtKey.VK_CONTROL] || true == m_KeyStates[(int) Win32.VirtKey.VK_LCONTROL] || true == m_KeyStates[(int) Win32.VirtKey.VK_RCONTROL])? true:false;
			}
		}


		internal Keys VKeyToKeys(int vKey)
		{
			int ReturnValue = vKey;
			if (true == m_Shift) 
			{
				ReturnValue |= 0x10000;
			}
			if (true == m_Control) 
			{
				ReturnValue |= 0x20000;
			}
			if (true == m_Alt) 
			{
				ReturnValue |= 0x40000;
			}
			return (Keys) ReturnValue;
		}

		internal char VKeyToAscii(int vKey)
		{
			//Add the key to the map

			//figure out which character to return
			char ReturnValue;
			switch (vKey)
			{
				case (int) Win32.VirtKey.VK_ESCAPE:
					ReturnValue = (char) 27;
					break;
				case (int) Win32.VirtKey.VK_SPACE:
					ReturnValue = ' ';
					break;
				case (int) Win32.VirtKey.VK_BACK:
					ReturnValue = (char) 8;
					break;
				case (int) Win32.VirtKey.VK_TAB:
					ReturnValue = (char) 9;
					break;
				case (int) Win32.VirtKey.VK_RETURN:
					ReturnValue = (char) 12;
					break;
				case (int) Win32.VirtKey.VK_OEM_1:
					if (true == m_Shift) 
					{
						ReturnValue = ':';
					} 
					else 
					{
						ReturnValue = ';';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_PLUS:
					ReturnValue = '+';
					break;
				case (int) Win32.VirtKey.VK_OEM_COMMA:
					ReturnValue = ',';
					break;
				case (int) Win32.VirtKey.VK_OEM_MINUS:
					ReturnValue = '-';
					break;
				case (int) Win32.VirtKey.VK_OEM_PERIOD:
					ReturnValue = '.';
					break;
				case (int) Win32.VirtKey.VK_OEM_2:
					if (true == m_Shift) 
					{
						ReturnValue = '?';
					} 
					else 
					{
						ReturnValue = '/';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_3:
					if (true == m_Shift) 
					{
						ReturnValue = '~';
					} 
					else 
					{
						ReturnValue = '`';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_4:
					if (true == m_Shift) 
					{
						ReturnValue = '{';
					} 
					else 
					{
						ReturnValue = '[';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_5:
					if (true == m_Shift) 
					{
						ReturnValue = '|';
					} 
					else 
					{
						ReturnValue = '\\';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_6:
					if (true == m_Shift) 
					{
						ReturnValue = '}';
					} 
					else 
					{
						ReturnValue = ']';
					}
					break;
				case (int) Win32.VirtKey.VK_OEM_7:
					if (true == m_Shift) 
					{
						ReturnValue = '\"';
					} 
					else 
					{
						ReturnValue = '\'';
					}
					break;
				case 0x30: //0
				case 0x31: //1
				case 0x32: //2
				case 0x33: //3
				case 0x34: //4
				case 0x35: //5
				case 0x36: //6
				case 0x37: //7
				case 0x38: //8
				case 0x39: //9
					if (true == m_Shift) 
					{
						ReturnValue = (char) (vKey - 15); //!@#$%^&*()
					} 
					else 
					{
						ReturnValue = (char) vKey;
					}
					break;
				case 0x41: //A
				case 0x42: //B
				case 0x43: //C
				case 0x44: //D
				case 0x45: //E
				case 0x46: //F
				case 0x47: //G
				case 0x48: //H
				case 0x49: //I
				case 0x4A: //J
				case 0x4B: //K
				case 0x4C: //L
				case 0x4D: //M
				case 0x4E: //N
				case 0x4F: //O
				case 0x50: //P
				case 0x51: //Q
				case 0x52: //R
				case 0x53: //S
				case 0x54: //T
				case 0x55: //U
				case 0x56: //V
				case 0x57: //W
				case 0x58: //X
				case 0x59: //Y
				case 0x5A: //Z
					if (true == m_Shift) 
					{
						ReturnValue = (char) vKey ;
					} 
					else 
					{
						ReturnValue = (char) (vKey + 32); //lower case
					}
					break;
				default:
					ReturnValue = (char) 0;
					break;
			} //switch
			return ReturnValue;
		} //input vkey function


		#region Keyboard Class Events

		/// <summary>Event fired when a key is pressed</summary>
		[Description("Event fired when a key is pressed on this keyboard")]
		public event SdgKeyEventHandler KeyDown;

		/// <summary>Event fired when a key is released</summary>
		[Description("Event fired when a key is released on this keyboard")]
		public event SdgKeyEventHandler KeyUp;

		/// <summary>Event fired when a key is pressed</summary>
		[Description("Event fired when a key is pressed on this keyboard")]
		public event SdgKeyPressEventHandler KeyPress;

		internal void OnKeyDown(SdgKeyEventArgs e)
		{
			if (KeyDown != null)
				KeyDown(this, e);
		}
		
		internal void OnKeyUp(SdgKeyEventArgs e)
		{
			if (KeyUp != null)
				KeyUp(this, e);
		}
		
		internal void OnKeyPress(SdgKeyPressEventArgs e)
		{
			if (KeyPress != null)
				KeyPress(this, e);
		}

		#endregion



		/// <summary>A class which inherits from ReadOnlyCollection Base and represents a collection of keyboards</summary>
		/// <remarks><para>To access different keyboards on the system a collection class was needed.  This class will 
		/// provide array access to the class and will provide a count of the total number of keyboards on the system.</para></remarks>
		[Description("A class which inherits from ReadOnlyCollection Base and represents a collection of keyboards")]
		public class KeyboardCollection : ReadOnlyCollectionBase
		{
			/// <summary>The default constructor for a keyboard collection.  Requires an arraylist of keyboards</summary>
			public KeyboardCollection(ArrayList value)
			{
				foreach (Object o in value)
				{
					InnerList.Add(o);
				}
			}

			/// <summary>Gets or sets the keyboard class specified by the index parameter</summary>
			/// <remarks><para>There are several ways to iterate through all of the keyboards on the system.
			/// One can use a numeric iterator such as "int i" or use a mouse iterator using the foreach parameter.</para></remarks>
			/// <example><para>C# Eg., for (int i=0; i &lt; sdgManager1.Keyboards.Count; ++i) { }</para></example>
			/// <example><para>C# Eg., foreach (Sdgt.Keyboard k in sdgManager1.Keyboards) { }</para></example>
			[Description("Gets or sets the keyboard class specified by the index parameter")]
			public Keyboard this[int index]
			{
				get
				{
					return (Keyboard) InnerList[index];
				}
				set
				{
					if (index == InnerList.Count)
					{
						InnerList.Add((object) value);
					}
					else 
					{
						InnerList[index] = (object) value;
					}
					
				}
			}

		
		}
	}

	#endregion

	#region Keyboard Event Code

	/// <summary>Keyboard event handler for key up and key down events</summary>
	[Description("Keyboard event handler for key up and key down events")]
	public delegate void SdgKeyEventHandler(object sender, SdgKeyEventArgs e);

	/// <summary>Keyboard event handler for key press events</summary>
	[Description("Keyboard event handler for key press events")]
	public delegate void SdgKeyPressEventHandler(object sender, SdgKeyPressEventArgs e);

	/// <summary>Event Arguments for the default keyboard press event</summary>
	[Description("Event Arguments for the default keyboard press event")]
	public class SdgKeyPressEventArgs : EventArgs
	{
		private int m_KeyboardID;
		private char m_KeyChar;
		private bool m_Handled;

		/// <summary>The Default Constructor for the key press event arguments</summary>
		public SdgKeyPressEventArgs()  { m_Handled = false; }

		/// <summary>Gets the ASCII equivalent of the character just pressed</summary>
		/// <remarks><para>Some keys such as f1 do not have ascii equivalents.  These keys
		/// will return null as the KeyChar value.  The Ascii Character takes into account 
		/// the use of the shift key and ignores the control and alt keys.</para></remarks>
		/// <example><para>C# Eg., if ('a' == e.KeyChar) {}</para></example>
		[Description("Gets the ASCII equivalent of the character just pressed")]
		public char KeyChar
		{
			get
			{
				return m_KeyChar;
			}
			set
			{
				m_KeyChar = value;
			}
		}

		/// <summary>Gets the ID of the keyboard which raised this event</summary>
		/// <remarks><para>This is the only property that is added from the normal .NET 
		/// keyboard event</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Keyboards[e.ID] </para></example>
		[Description("Gets the ID of the keyboard which raised this event")]
		public int ID
		{
			get
			{
				return m_KeyboardID;
			}
			set
			{
				m_KeyboardID = value;
			}
		}

		/// <summary>Gets a boolean value describing if this event has been handled by another component</summary>
		/// <remarks><para>Sometimes a keyboard event could be handled by a text box or form.  You are told wether or
		/// not this keyboard event has been handled by another control through the handled property.</para></remarks>
		/// <example><para>C# Eg., if (false == e.Handled) {} </para></example>
		[Description("Gets a boolean value describing if this event has been handled by another component")]
		public bool Handled
		{
			get
			{
				return m_Handled;
			}
			set
			{
				m_Handled = value;
			}
		}

	}

	/// <summary>Event Arguments for the default keyboard down and up events</summary>
	[Description("Event Arguments for the default keyboard down and up events")]
	public class SdgKeyEventArgs : EventArgs
	{
		private bool m_Alt;
		private bool m_Control;
		private bool m_Handled;
		private bool m_Shift;
		private int m_KeyboardID;
		private Keys m_KeyCode;
		private Keys m_KeyData;
		private int m_KeyValue;
		private Keys m_Modifiers;

		/// <summary>The Default Constructor for the key up and down event arguments</summary>
		public SdgKeyEventArgs()
		{
			m_Handled = false;
		}

		/// <summary>Gets a boolean value representing the current state of the Alt key</summary>
		/// <remarks><para>This is a simple utility to determine if the either or both of the
		/// Alt keys are pressed down.</para></remarks>
		/// <example><para>C# Eg., if (true == e.Alt) {}</para></example>
		[Description("Gets a boolean value representing the current state of the Alt key")]
		public bool Alt
		{
			get
			{
				return m_Alt;
			}
			set
			{
				m_Alt = value;
			}
		}

		/// <summary>Gets a boolean value representing the current state of the Control key</summary>
		/// <remarks><para>This is a simple utility to determine if the either or both of the
		/// Control keys are pressed down.</para></remarks>
		/// <example><para>C# Eg., if (true == e.Control) {}</para></example>	
		[Description("Gets a boolean value representing the current state of the Control key")]
		public bool Control
		{
			get
			{
				return m_Control;
			}
			set
			{
				m_Control = value;
			}
		}

		/// <summary>Gets a boolean value describing if this event has been handled by another component</summary>
		/// <remarks><para>Sometimes a keyboard event could be handled by a text box or form.  You are told wether or
		/// not this keyboard event has been handled by another control through the handled property.</para></remarks>
		/// <example><para>C# Eg., if (false == e.Handled) {} </para></example>		
		[Description("Gets a boolean value describing if this event has been handled by another component")]
		public bool Handled
		{
			get
			{
				return m_Handled;
			}
			set
			{
				m_Handled = value;
			}
		}

		/// <summary>Gets a boolean value representing the current state of the Shift key</summary>
		/// <remarks><para>This is a simple utility to determine if the either or both of the
		/// Shift keys are pressed down.  The caps lock key is also taken into consideration for this value</para></remarks>
		/// <example><para>C# Eg., if (true == e.Shift) {}</para></example>
		[Description("Gets a boolean value representing the current state of the Shift key")]
		public bool Shift
		{
			get
			{
				return m_Shift;
			}
			set
			{
				m_Shift = value;
			}
		}

		/// <summary>Gets the ID of the keyboard which raised this event</summary>
		/// <remarks><para>This is the only property that is added from the normal .NET 
		/// keyboard event</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Keyboards[e.ID] </para></example>
		[Description("Gets the ID of the keyboard which raised this event")]
		public int ID
		{
			get
			{
				return m_KeyboardID;
			}
			set
			{
				m_KeyboardID = value;
			}
		}

		/// <summary>Gets the Keys enumeration of the current key pressed without modifiers such as Alt, Control and Shift</summary>
		/// <remarks><para>If you would like to know which key is currently being pressed this property provides this functionality</para></remarks>
		/// <example><para>C# Eg., if (Keys.Right == e.KeyCode) {}</para></example>
		[Description("Gets the Keys enumeration of the current key pressed without modifiers such as Alt, Control and Shift")]
		public Keys KeyCode
		{
			get
			{
				return m_KeyCode;
			}
			set
			{
				m_KeyCode = value;
			}
		}

		/// <summary>Gets the Keys enumeration of the current key pressed with modifiers such as Alt, Control and Shift</summary>
		/// <remarks><para>This functin provides you with a list of all the keys currently pressed.</para></remarks>
		/// <example><para>C# Eg. if ((Keys.Shift &amp; e.KeyData) > 0 &amp;&amp; (Keys.Right &amp; e.KeyData) > 0) {}</para></example>
		[Description("Gets the Keys enumeration of the current key pressed with modifiers such as Alt, Control and Shift")]
		public Keys KeyData
		{
			get
			{
				return m_KeyData;
			}
			set
			{
				m_KeyData = value;
			}
		}

		/// <summary>Gets a numeric value representing the acsii character of the key pressed</summary>
		/// <remarks><para>This is a quick and easy method of determining the Ascii character that would
		/// have been produced if this key was typed in a text box.</para></remarks>
		/// <example><para>C# Eg., if ((int) 'a' == e.KeyValue) {}</para></example>
		[Description("Gets a numeric value representing the acsii character of the key pressed")]
		public int KeyValue
		{
			get
			{
				return m_KeyValue;
			}
			set
			{
				m_KeyValue = value;
			}
		}

		/// <summary>Returns a Keys enumeration of all the modifiers used (such as Alt, Control and Shift) in the key event</summary>
		/// <remarks><para>This is useful for telling you which of Control, Alt and Shift are current depressed</para></remarks>
		/// <example><para>C# Eg., if ((Keys.Shift &amp; e.Modifiers) > 0 &amp;&amp; (Keys.Control &amp; e.Modifiers) > 0) {}</para></example>
		[Description("Returns a Keys enumeration of all the modifiers used (such as Alt, Control and Shift) in the key event")]
		public Keys Modifiers
		{
			get
			{
				return m_Modifiers;
			}
			set
			{
				m_Modifiers = value;
			}
		}

		
	}

	#endregion
}