#define SHOWCURSORHOTSPOT

using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;




namespace Sdgt
{
	/// <summary>The cardinal directions enumeration is useful for rotating the cursor in specific directions.  It assumes a default cursor rotation</summary>
	[Description("The cardinal directions enumeration is useful for rotating the cursor in specific directions.  It assumes a default cursor rotation")]
	public enum CardinalDirections
	{
		/// <summary>Normal</summary>
		Normal = 0,
		/// <summary>North</summary>
		North = 25,
		/// <summary>North East</summary>
		NorthEast = 70,
		/// <summary>East</summary>
		East = 115, 
		/// <summary>South East</summary>
		SouthEast = 160, 
		/// <summary>South</summary>
		South = 205, 
		/// <summary>South West</summary>
		SouthWest = 250, 
		/// <summary>West</summary>
		West = 295, 
		/// <summary>North West</summary>
		NorthWest = -20 
		
	}

	/// <summary>An sdg mouse class, this class handles all mouse movements and fires events specific to this mouse</summary>
	/// <remarks><para>An sdg mouse is a window which moves when it's X and Y values are changed.  It handles cursor 
	/// movement and rotation.  It is possible to create a dummy sdg mouse which you can control manually by using the 
	/// constructor that does require an IntPtr as the first variable</para></remarks>
	[Description("An sdg mouse class, this class handles all mouse movements and fires events specific to this mouse")]
	[DefaultEvent("MouseMove")]
	public class Mouse
	{
		/// <summary>The Default Mouse Constructor.  Requires a pointer to a mouse</summary>
		public Mouse(IntPtr value, int x, int y)
		{
			
			impl = new MouseImpl(value, x, y);
		}

		/// <summary>Specialized Mouse Constructor used if you want to create a dummy mouse that just moves around the screen</summary>
		public Mouse(int x, int y)
		{
			impl = new MouseImpl(IntPtr.Zero, x, y);
			
		}

		/// <summary>Get or sets the Hotspot of the cursor.  The hotspot is the focal area of a cursor</summary>
		/// <remarks><para>Changing the hotspot is usually the result of an error on my part.  However,
		/// if you decide to create your own cursor and don't customize your own cursor this function may
		/// be of great use to you.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Hotspot = new Point(0,0);</para></example>
		[Description("Get or sets the Hotspot of the cursor.  The hotspot is the focal area of a cursor")]
		public Point Hotspot
		{
			get
			{
				return impl.myHotspot;
			}
			set
			{
				impl.myHotspot = value;
			}
		}

		/// <summary>Gets or sets the rotation of the cursor image.  This does not affect the movement of the cursor</summary>
		/// <remarks><para>There may be the case where you want to rotate just the cursor image without rotating the 
		/// movement of the cursor.  This utility provides that functionality.  Please note that this can also be used 
		/// with the CardinalDirections enumeraton. </para></remarks> 
		/// <example><para>C# Eg., sdgManager1.Mice[0].RotateCursorImageOnly = (int) CardinalDirections.South</para></example>
		/// <example><para>C# Eg., sdgManager1.Mice[0].RotateCursorImageOnly = 90;</para></example>
		
		[Description("Gets or sets the rotation of the cursor image.  This does not affect the movement of the cursor")]
		public int RotateCursorImageOnly
		{
			get
			{
				return impl.RotateCursorImageOnly;
			}
			set
			{
				impl.RotateCursorImageOnly = value;
			}
		}

		/// <summary>Gets or sets both the rotation of the cursor image and the rotation of the movement of the cursor.  For table top displays</summary>
		/// <remarks><para>Table Top applications often require the rotation of cursors and cursor movement.  When one is rotated 180 from the 
		/// upright screen position moving the mouse awat from the user will move the cursor towards the user instead of away from the user.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].DegreeRotation = 180;</para></example>
		[Description("Gets or sets both the rotation of the cursor image and the rotation of the movement of the cursor.  For table top displays")]
		public int DegreeRotation
		{
			get
			{
				return impl.DegreeRotation;
			}
			set
			{
				impl.DegreeRotation = value;
			}
		}

		/// <summary>Gets or sets the visible property of the Mouse.  This is useful if you just want Mouse Events but you don't want to draw the Mouse on the screen</summary>
		/// <remarks><para>If a user adds mutliple sdgManagers to a form they may not want multiple cursors for each sdgManager.  This utility provides the means to
		/// prevent the drawing of another cursor on the screen.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Visible = false; </para></example>
		[Description("Gets or sets the visible property of the Mouse.  This is useful if you just want Mouse Events but you don't want to draw the Mouse on the screen")]
		public bool Visible 
		{
			get
			{
				return impl.m_fVisible;
			}
			set
			{
				impl.m_fVisible = value;
				//impl.Opacity = (true == value)?1.0:0.001;
				impl.Visible = value;
			}
		}

		/// <summary>Gets or sets the cursor image.  You can use one of the default System.Windows.Forms.Cursors</summary>
		/// <remarks><para>Often in an sdg application you may want to customize the cursor for different users of
		/// the application.  The cursor property allows you to use the default windows cursors and your own custom
		/// .cur files.  Please note that animated cursors and inverted colours are currently not supported.</para>
		/// <para>There have been some issues noted with the loading of cursors that are not black and white and 32x32 pixels, 
		/// unfortunately these problems are systemic in the .NET framework implementation of the cursor class.  We expect this issue to
		/// be solved but in the mean time to create a cursor in Visual Studio .NET go File->New->File->Visual C++->Cursor File.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Cursor = System.Windows.Forms.Cursors.Cross;</para></example>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Cursor = new Cursor(@"c:\windows\cursors\larrow.cur");</para></example>
		[Description("Gets or sets the cursor image.  You can use one of the default System.Windows.Forms.Cursors")]
		public Cursor Cursor
		{
			get
			{
				return impl.myCursor;		
			}
			set
			{
				impl.myCursor = value;
			}
		}

		

		/// <summary>Gets or sets the absolute X screen position of the current Mouse</summary>
		/// <remarks><para>Often it may be useful to obtain the absolute X position of the cursor
		/// without the need to worry about it being relative to the form.  Xabs provides this functionality</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Xabs = 0;</para></example>
		[Description("Gets or sets the absolute X screen position of the current Mouse")]
		public int Xabs
		{
			get	
			{
				int retvalue = X;
				if (null != impl.RelativeTo) 
					retvalue = retvalue + impl.myClientRect.Left;
				return retvalue;  
			}
			set					
			{
				int newvalue = value;
				if (null != impl.RelativeTo) 
					newvalue = newvalue - impl.myClientRect.Left;
				X = newvalue; 
			}
		}

		/// <summary>Gets or sets the absolute Y screen position of the current Mouse</summary>
		/// <remarks><para>Often it may be useful to obtain the absolute Y position of the cursor
		/// without the need to worry about it being relative to the form.  Yabs provides this functionality</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Yabs = 0;</para></example>
		[Description("Gets or sets the absolute Y screen position of the current Mouse")]
		public int Yabs
		{
			get
			{
				int retvalue = Y;
				if (null != impl.RelativeTo) 
					retvalue = retvalue + impl.myClientRect.Top;
				return retvalue; 
			}
			set					
			{
				int newvalue = value;
				if (null != impl.m_RelativeControl) 
					newvalue = newvalue - impl.myClientRect.Top;
				Y = newvalue; 
			}
		}

		/// <summary>Gets or sets the absolute screen position of the current Mouse</summary>
		/// <remarks><para>Often it may be useful to obtain the absolute position of the cursor
		/// without the need to worry about it being relative to the form.  LocationAbs provides this functionality</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].LocationAbs = new Point(0,0);</para></example>
		[Description("Gets or sets the absolute screen position of the current Mouse")]
		public Point LocationAbs
		{
			get
			{
				
				return new Point(Xabs, Yabs);
			}
			set
			{
				
				Xabs = value.X;
				Yabs = value.Y;
			}
		}

		/// <summary>Gets or sets the X position of the cursor relative to the specified control</summary>
		/// <remarks><para>X provides the same coordinates that are provided in the Mouse Event.  You
		/// can move to mouse to a particular location by setting the mouse value.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].X = 0;</para></example>
		[Description("Gets or sets the X position of the cursor relative to the specified control")]
		public int X
		{
			get
			{ 
				double x = (double) impl.myHotspot.X * impl.m_dRotationMatrix[0] + (double) impl.myHotspot.Y * impl.m_dRotationMatrix[2];
				int retvalue = impl.X + (int) x + impl.myTranslation.X;
				if (null != impl.RelativeTo)	
				{
					retvalue = retvalue - impl.myClientRect.Left;
				}
				if (impl.DegreeRotation == 0)
				{
					retvalue = retvalue - impl.myHotspot.X;
				}
				return retvalue; 
			}
			set					
			{
				int newValue = value - impl.myHotspot.X - impl.myTranslation.X; 
				if (null != impl.RelativeTo)	
				{
					newValue = newValue + impl.myClientRect.Left;
				}
				if (impl.DegreeRotation == 0)
				{
					newValue = newValue + impl.myHotspot.X;
				}
				impl.X = newValue; 
			}
		}

		/// <summary>Gets or sets the Y position of the cursor relative to the specified control</summary>
		/// <remarks><para>Y provides the same coordinates that are provided in the Mouse Event.  You
		/// can move to mouse to a particular location by setting the mouse value.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Y = 0;</para></example>
		[Description("Gets or sets the Y position of the cursor relative to the specified control")]
		public int Y
		{
			get
			{
				double y = (double) impl.myHotspot.X * impl.m_dRotationMatrix[1] + (double) impl.myHotspot.Y * impl.m_dRotationMatrix[3];
				int retvalue = impl.Y + (int) y + impl.myTranslation.Y;
				if (null != impl.m_RelativeControl)	
				{
					retvalue = retvalue - impl.myClientRect.Top;
				}
				return retvalue; 
			}	
			set
			{ 
				int newValue = value - impl.myHotspot.Y - impl.myTranslation.Y; 
				if (null != impl.RelativeTo)	
				{
					value = value + impl.myClientRect.Top;
				}
				impl.Y = newValue; 
			}

		}

		/// <summary>Gets or sets the position of the cursor relative to the specified control</summary>
		/// <remarks><para>Location provides the same coordinates that are provided in the Mouse Event.  You
		/// can move to mouse to a particular location by setting the mouse value.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Location = new Point(0,0);</para></example>
		[Description("Gets or sets the position of the cursor relative to the specified control")]
		public Point Location
		{
			get
			{
				return new Point(X,Y);
			}
			set
			{
				X = value.X;
				Y = value.Y;
			}
			

		}

		/// <summary>Gets or sets the control that all of the X, Y and Location coordinates are relative to</summary>
		/// <remarks><para>This function makes all of the cursor coordinates relative to a particular part of 
		/// the screen.  This function allows a user to make their coordinates relative to a particular control.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].RelativeTo = this;</para></example>
		
		[Description("Gets or sets the control that all of the X, Y and Location coordinates are relative to")]
		public Control RelativeTo
		{
			get
			{
				return impl.RelativeTo;
			}
			set
			{
				if (null != impl.RelativeTo)
				{
					impl.RelativeTo.Resize -= new EventHandler(RelativeFormResized);	
				}
				impl.RelativeTo = value;
				if (null != impl.RelativeTo)
				{
					impl.RelativeTo.Resize += new EventHandler(RelativeFormResized);
					Point p = value.PointToScreen(new Point(0,0));
					impl.myClientRect = new Rectangle(p,value.Size);
				}
				

			}
		}

		private void RelativeFormResized(object sender, EventArgs e)
		{
			Point p = impl.RelativeTo.PointToScreen(new Point(0,0));
			impl.myClientRect = new Rectangle(p,impl.RelativeTo.Size);
		}

		/// <summary>Gets or sets the cursor caption.  The Default text location is the bottom left of the screen</summary>
		/// <remarks><para>To customize a cursor (with a name or identifier) simply add text to the text property.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Text = "Silly Rabbit";</para></example>
		[Description("Gets or sets the cursor caption.  The Default text location is the bottom left of the screen")]
		public String Text
		{
			get
			{
				return impl.myText;
			}
			set
			{
				impl.myText = value;
			}
		}

		/// <summary>Gets or sets the cursor caption string format.  The default format is to draw all text in a left to right fashion moving right from TextLocation</summary>
		/// <remarks><para>It may be useful to orient the text with different alignments (left, center, right).  The StringFormat class provides the ability to start and move
		/// in different directions.  For more information see the StringFormat class.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].TextFormat = new StringFormat(StringFormatFlags.DirectionRightToLeft);</para></example>
		[Description("Gets or sets the cursor caption string format.  The default format is to draw all text in a left to right fashion moving right from TextLocation")]
		public StringFormat TextFormat 
		{
			get
			{
				return impl.m_TextFormat;
			}
			set
			{
				impl.m_TextFormat = value;
				impl.ResizeWindow();
				impl.Refresh();
			}
		}

		/// <summary>Gets or sets the cursor caption brush.  This is useful for changing the colour of the text</summary>
		/// <remarks><para>To change the colour of the text or change the way text is drawn on the screen one may want to set the cursor caption brush.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].TextBrush = new SolidBrush(Color.Blue);</para></example>
		[Description("Gets or sets the cursor caption brush.  This is useful for changing the colour of the text")]
		public SolidBrush TextBrush
		{
			get
			{
				return impl.m_Brush;
			}
			set
			{
				
				impl.m_Brush = value;
				impl.ResizeWindow();
				impl.Refresh();
			}
		}

		/// <summary>Gets or sets the starting location of the text caption for this cursor.</summary>
		/// <remarks><para>This command allows the programmer to change the starting location of the text caption.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].TextLocation = new Point(30,30);</para></example>
		[Description("Gets or sets the starting location of the text caption for this cursor.")]
		public Point TextLocation
		{
			get
			{
				return impl.m_TextLocation;
			}
			set
			{
				impl.m_TextLocation = value;
				impl.ResizeWindow();
				impl.Refresh();
				
			}
		}

		/// <summary>Gets or sets the font of the text caption for this cursor.</summary>
		/// <remarks><para>This command allows the programmer to change the font type and size of the text caption.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].TextFont = new Font("Times New Roman", 12);</para></example>
		[Description("Gets or sets the font of the text caption for this cursor.")]
		public Font TextFont
		{
			get
			{
				return impl.m_Font;
				
			}
			set
			{
				impl.m_Font = value;
				impl.ResizeWindow();
				impl.Refresh();
			}
		}

		/// <summary>Gets or sets the position of the text caption based upon the Cardinal Direction enumeration.  Normal returns a value dead center of the cursor.</summary>
		/// <remarks><para>To change the text cursor to various different locations relative to the cursor.  If text is located on the northwest part of the cursor then
		/// a programmer will probably want the text right justified and relative to the top left corner.  The Cardinal Position enumeration provides a base set of 
		/// text locations.  More custom positions can be produced using the TextFormat and TextLocation properties.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].TextCardinalPosition = Sdgt.CardinalDirections.South;</para></example>
		[Description("Gets or sets the position of the text caption based upon the Cardinal Direction enumeration.  Normal returns a value dead center of the cursor.")]
		public CardinalDirections TextCardinalPosition
		{
			get
			{
				return impl.myTextCardinalPosition; 
			}
			set
			{
				impl.myTextCardinalPosition = value;
			}
		}

		/// <summary>Gets or sets the opacity of the cursor.  A value of 1.0 represents full opacity and 0.0 represents completely transparent</summary>
		/// <remarks><para>It may be useful to make a cursor more transparent to avoid occulsion effects from a cursor.  Opacity changes the cursor 
		/// transparency from fully opaque (1.0) to fully transparent (0.0).  Please note that opacity has been known to dramatically decrease mouse
		/// performance on machines without hardware accelerated transparent windows.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Opacity = 0.5;</para></example>
		[Description("Gets or sets the opacity of the cursor.  A value of 1.0 represents full opacity and 0.0 represents completely transparent")]
		public double Opacity
		{
			get
			{
				return impl.Opacity;
			}
			set
			{
				impl.Opacity = value;
			}
		}
		
		internal void setButtonState(int index, bool value)
		{
			if (index >= 0 && index <= Constants.MAXMOUSEBUTTONS)
			{
			
				impl.m_fButtonState[index] = value;
			}
		}

		/// <summary>Gets an enumeration of all the mouse buttons currently pressed down</summary>
		/// <remarks><para>The mouse class keeps track of which buttons are pressed down.  
		/// Please note that it is possible for more than one mouse button to be pressed
		/// at any given time.</para></remarks>
		/// <example><para>C# Eg., if (sdgManager1.Mice[0].Buttons &amp; MouseButtons.Left) {} </para></example>
		[Description("Gets an enumeration of all the mouse buttons currently pressed down")]
		public MouseButtons Button
		{
			get
			{
				return impl.Button;
			}
		}

		


		/// <summary>Gets or sets the current SDG widget that this mouse is focused on</summary>
		/// <remarks><para>Each mouse has it's own focus.  When a keyboard event occurs, the
		/// SDG manager recieves focus from the widget</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Focus = (ISdgMouseWidget) label1;</para></example>
		[Description("Gets or sets the current SDG widget that this mouse is focused on")]
		public ISdgMouseWidget Focus
		{
			get
			{
				return impl.m_Widget;
			}
			set
			{
				impl.m_Widget = value;
			}
		}

		/// <summary>Gets or sets the current Control that this mouse is focused on</summary>
		/// <remarks><para>Each mouse has it's own Control focus.  This utility allows interaction with normal 
		/// widgets as well as SDG Widgets through the standard SDG events.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[0].Focus = (ISdgMouseWidget) label1;</para></example>
		[Description("Gets or sets the current Control that this mouse is focused on")]
		public Control ControlFocus
		{
			get
			{
				return impl.m_Control;
			}
			set
			{
				impl.m_Control = value;
			}
		}


		internal int MouseButtonsOnWidget
		{
			get
			{
				return impl.m_MouseButtonsOnWidget;
			}
			set
			{
				impl.m_MouseButtonsOnWidget = value;
				
			}


		}

		internal class MouseImpl : Form
		{
			#region Mouse Impl Variables

			public bool m_RotateDelta = false;
			public Point m_Location = new Point(500,500);
			public Point m_ShiftCursorAmount = new Point(0,0);
			public bool m_fVisible = true;
			public IntPtr m_hDevice;
			public Point m_TextLocation = new Point(30,30);
			public int m_MouseButtonsOnWidget;

			public int m_iLastMouseButton;
			public int m_DegreeRotation;
			public Control m_Control;
			public ISdgMouseWidget m_Widget;
			public int m_iFrameIndex;
			public int m_iTotalFrames;
			
			
			
			public Point m_lastPoint;
			public Point m_Hotspot;
			public IntPtr m_hCursor = IntPtr.Zero;
			
			public Cursor m_Cursor;
			public double[] m_dRotationMatrix = {1.0, 0.0, 0.0, 1.0};
		

			public Font  m_Font;
			public StringFormat m_TextFormat;
			public CardinalDirections m_TextCardinal;

			public SolidBrush m_Brush;
			public String m_Text = null; 
			public String m_sCursor = null;

			public Control m_RelativeControl;
			public bool[] m_fButtonState = new bool[Constants.MAXMOUSEBUTTONS + 1];
			public Rectangle m_ClientRect;

			internal System.Threading.Timer m_BoundsTimer;

			#endregion



			public MouseImpl(IntPtr value, int x, int y)
			{
				m_hDevice = value;
				m_Location = new Point(x,y);
				InitializeMouse();
				
			}

			public MouseImpl(int x, int y)
			{
				m_hDevice = IntPtr.Zero;
				m_Location = new Point(x,y);
				InitializeMouse();
			}

			public Rectangle myClientRect
			{
				get
				{
					return m_ClientRect;
				}
				set
				{
					m_ClientRect = value;
				}
			}

			public Point myTranslation
			{
				get
				{
					return m_ShiftCursorAmount;
				}
				set
				{
					m_ShiftCursorAmount = value;
				}
			}

			public Point myHotspot
			{
				get
				{
					return m_Hotspot;
				}
				set
				{
					m_Hotspot = value;
				}
			}


			public MouseButtons Button
			{
				get
				{
					MouseButtons value = MouseButtons.None;
					if (m_fButtonState[0] == true)	
					{
						value = (MouseButtons) (value | MouseButtons.Left);
					}
					if (m_fButtonState[1] == true) 
					{
						value = (MouseButtons) (value | MouseButtons.Right);
					}
					if (m_fButtonState[2] == true) 
					{
						value = (MouseButtons) (value | MouseButtons.Middle);
					}
					if (m_fButtonState[3] == true) 
					{
						value = (MouseButtons) (value | MouseButtons.XButton1);
					}
					if (m_fButtonState[4] == true) 
					{
						value = (MouseButtons) (value | MouseButtons.XButton2);
					}
					return  value; 
				}

			

		}


			public Rectangle ClientRect
			{
				get
				{
					return m_ClientRect;
				}
				set
				{
					m_ClientRect = value;
				}
			}

			public Control RelativeTo
			{
				get
				{
					return m_RelativeControl;
				}
				set
				{
					m_RelativeControl = value;
				}

			}



			public int X
			{
				get
				{
					//double x = (double) m_Hotspot.X * m_dRotationMatrix[0] + (double) m_Hotspot.Y * m_dRotationMatrix[2];
					//double xShift = (double) m_ShiftCursorAmount.X * m_dRotationMatrix[0] + (double) m_ShiftCursorAmount.Y * m_dRotationMatrix[2];
					return m_Location.X; // + (int) x;// + (int) m_ShiftCursorAmount.X;
					
					
					
				}
				set
				{
					//double x = (double) m_Hotspot.X * m_dRotationMatrix[0] + (double) m_Hotspot.Y * m_dRotationMatrix[2];
					//double xShift = (double) m_ShiftCursorAmount.X * m_dRotationMatrix[0] + (double) m_ShiftCursorAmount.Y * m_dRotationMatrix[2];
					if (CheckBounds(value, m_Location.Y))	
					{
						m_Location.X = value;// - (int) x;// + (int) m_ShiftCursorAmount.X;
						if (m_fVisible)
						{
							Location = m_Location;
							Refresh();
						}
					}
				}

			}

			public int Y
			{
				get
				{
					//double y = (double) m_Hotspot.X * m_dRotationMatrix[1] + (double) m_Hotspot.Y * m_dRotationMatrix[3];
					//double yShift = (double) m_ShiftCursorAmount.X * m_dRotationMatrix[1] + (double) m_ShiftCursorAmount.Y * m_dRotationMatrix[3];
					return m_Location.Y;// + (int) y;// - (int) m_ShiftCursorAmount.Y;
				}
				set
				{
					//double y = (double) m_Hotspot.X * m_dRotationMatrix[1] + (double) m_Hotspot.Y * m_dRotationMatrix[3];
					//double yShift = (double) m_ShiftCursorAmount.X * m_dRotationMatrix[1] + (double) m_ShiftCursorAmount.Y * m_dRotationMatrix[3];
					if (CheckBounds(m_Location.X, value))	
					{
						m_Location.Y = value;// - (int) y;// + (int) m_ShiftCursorAmount.Y;
						if (m_fVisible)
						{
							Location = m_Location;
							Refresh();
						}
					}
					
				}

			}


			public void ChangeXY(int newX, int newY, int MouseX, int MouseY)
			{
				ChangeXY(newX,newY);
				Win32.SetCursorPos(MouseX,MouseY);
			}

			public void ChangeXY(int newX, int newY)
			{
				double x;
				double y;

				if (true == m_RotateDelta)
				{
					x = (double) newX * m_dRotationMatrix[0] + (double) newY * m_dRotationMatrix[2];
					y = (double) newX * m_dRotationMatrix[1] + (double) newY * m_dRotationMatrix[3];
				} 
				else 
				{ //Rotate image but not delta
					x = newX;
					y = newY;
				}

				if (CheckBounds(m_Location.X + (int) x + m_ShiftCursorAmount.X, m_Location.Y + (int) y + m_ShiftCursorAmount.Y))
				{
					m_Location = new Point(m_Location.X + (int) x, m_Location.Y + (int) y);
				}
				else 
				{
					if (true == CheckBounds(m_Location.X + (int) x + m_ShiftCursorAmount.X, 50))
					{ // x is ok
						m_Location.X = m_Location.X + (int) x;

					}		
					if (true == CheckBounds(50, m_Location.Y + (int) y + m_ShiftCursorAmount.Y))
					{ // y is ok
						m_Location.Y = m_Location.Y + (int) y;
					}
					

				}


				if ((newX != 0 || newY != 0)&&(true == m_fVisible))
				{
					Location = m_Location;
				}
			}

			bool CheckBounds(int X, int Y)
			{

				if (SystemInformation.VirtualScreen.Contains(X,Y))
				{
					return true;
				} 
				else 
				{
					return false;
				}

				

			}

			public void InitializeMouse()
			{
				m_ClientRect = new Rectangle(0,0,1,1);
		
				Win32.ICONINFO iconinfo = new Win32.ICONINFO();
				Win32.GetIconInfo(Cursors.Default.Handle, ref iconinfo);
				m_Hotspot = new Point(iconinfo.xHotspot, iconinfo.yHotspot);
				m_Widget = null;
				m_MouseButtonsOnWidget = 0;
				
				m_lastPoint = new Point(0,0);
				m_DegreeRotation = 360;

				//Preparation of Fonts
				m_Font = new Font("Arial", 10);
				m_Brush = new SolidBrush(Color.Black);
				m_Text = null; 
				m_TextLocation = new Point(20,30);
				m_TextFormat = new StringFormat();
				m_TextFormat.Alignment = StringAlignment.Near;
				m_TextFormat.LineAlignment = StringAlignment.Near;
				m_TextFormat.Trimming = StringTrimming.None;
				m_TextFormat.FormatFlags = StringFormatFlags.NoClip | StringFormatFlags.NoWrap;
				
		
				m_iTotalFrames = 1;
				m_iFrameIndex = 0;
				for (int i=0; i < Constants.MAXMOUSEBUTTONS+1; ++i) 
				{
					m_fButtonState[i] = false;
				}

				//Form Settings
				BackColor = Color.AliceBlue;
				TransparencyKey = Color.AliceBlue;
				ForeColor = Color.Black;
				Cursor = null;
				TopMost = true;
				Capture = false;
				
				
				FormBorderStyle = FormBorderStyle.None;
				ShowInTaskbar = false;
			
				//Default Cursor Loading
				m_Cursor = Cursors.Arrow;
				Size = m_Cursor.Size;
				Win32.ShowCursor(false);
				myTextCardinalPosition = CardinalDirections.SouthEast;
			
				//Positioning
				m_iLastMouseButton = 0;
			
				Show();
				Location = m_Location;
				ResizeWindow();
				Refresh();
				Enabled = false;

				m_BoundsTimer = new System.Threading.Timer(new System.Threading.TimerCallback(BoundsTimer_Tick), null, 0, 500);
			}

			public void BoundsTimer_Tick(object state)
			{

				if (m_Location.X > SystemInformation.VirtualScreen.Right)
				{
					m_Location.X = SystemInformation.VirtualScreen.Right;	
				}
				else if (m_Location.X < SystemInformation.VirtualScreen.Left)
				{
					m_Location.X = SystemInformation.VirtualScreen.Left;
				}
				if (m_Location.Y > SystemInformation.VirtualScreen.Bottom)
				{
					m_Location.Y = SystemInformation.VirtualScreen.Bottom;
				}
				else if (m_Location.Y < SystemInformation.VirtualScreen.Top)
				{
					m_Location.Y = SystemInformation.VirtualScreen.Top;
				}
					
			}
			public void ResizeWindow()
			{
				//Resize the window to find out what the size is

				Point tl = new Point(0,0);
				Point br = new Point(0,0);
				Point[] boundingRectangle = new Point[4];
				Rectangle[] myRect = new Rectangle[2];

				//size of cursor
				if (null != m_Cursor) 
				{
					myRect[0].Location = new Point(0, 0);
					myRect[0].Size = new Size(m_Cursor.Size.Width, m_Cursor.Size.Height);
				} 
				else 
				{
					myRect[0].Location =  new Point(0,0);
					myRect[0].Size =  new Size(32,32); //largest size possible by Cursor Dance.
				}
				//size of Text
				if (null == m_Text)	
				{
					myRect[1].Location = new Point(0,0);
					myRect[1].Size = new Size(1,1);
				} 
				else 
				{
					myRect[1].Location = m_TextLocation;
					Graphics graphics = this.CreateGraphics();
					SizeF stringSize;
					stringSize = graphics.MeasureString(m_Text, m_Font, new PointF((float) m_TextLocation.X, (float) m_TextLocation.Y), m_TextFormat);
					Point myPoint = new Point(m_TextLocation.X,m_TextLocation.Y);
					if ((StringFormatFlags.DirectionRightToLeft & m_TextFormat.FormatFlags)>0)
					{
						myPoint.X = myPoint.X - (int) stringSize.Width;
					}
					if (StringAlignment.Center == m_TextFormat.Alignment)
					{
						myPoint.X = myPoint.X - (int) (stringSize.Width / 2.0);
					}
					if ((StringAlignment.Far == m_TextFormat.LineAlignment))
					{
						myPoint.Y = myPoint.Y - (int) stringSize.Height;
					}
					myRect[1].Location = myPoint;
					myRect[1].Size = new Size((int) stringSize.Width, (int) stringSize.Height);
					graphics.Dispose();

				}
				//find the unrotated bounding rectangle
				for (int i=0; i< myRect.Length; ++i)
				{
					if (0 == i)
					{
						tl = new Point(myRect[0].Left, myRect[0].Top);
						br = new Point(myRect[0].Right, myRect[0].Bottom);
					} 
					else 
					{
						if (myRect[i].Left < tl.X) 
						{
							tl.X = myRect[i].Left;
						} 
						if (myRect[i].Top < tl.Y) 
						{
							tl.Y = myRect[i].Top;
						}
						if (myRect[i].Right > br.X) 
						{
							br.X = myRect[i].Right;
						}
						if (myRect[i].Bottom > br.Y) 
						{
							br.Y = myRect[i].Bottom;
						}
					}
				}
				//perform the matrix rotation
				//tl tr bl br
				double x1 = (double) tl.X * m_dRotationMatrix[0] + (double) tl.Y * m_dRotationMatrix[2];
				double x2 = (double) br.X * m_dRotationMatrix[0] + (double) br.Y * m_dRotationMatrix[2];
				double x3 = (double) br.X * m_dRotationMatrix[0] + (double) tl.Y * m_dRotationMatrix[2];
				double x4 = (double) tl.X * m_dRotationMatrix[0] + (double) br.Y * m_dRotationMatrix[2];

				double y1 = (double) tl.X * m_dRotationMatrix[1] + (double) tl.Y * m_dRotationMatrix[3];
				double y2 = (double) br.X * m_dRotationMatrix[1] + (double) br.Y * m_dRotationMatrix[3];
				double y3 = (double) br.X * m_dRotationMatrix[1] + (double) tl.Y * m_dRotationMatrix[3];
				double y4 = (double) tl.X * m_dRotationMatrix[1] + (double) br.Y * m_dRotationMatrix[3];

				boundingRectangle[0] = new Point((int) x1,(int) y1);
				boundingRectangle[1] = new Point((int) x3,(int) y3);
				boundingRectangle[2] = new Point((int) x4,(int) y4);
				boundingRectangle[3] = new Point((int) x2,(int) y2);

				Point mostTL = new Point(0,0);
				Point mostBR = new Point(0,0);
				for (int i=0; i< boundingRectangle.Length; i++)
				{
					if (0 == i) 
					{
						mostTL = boundingRectangle[0];
						mostBR = boundingRectangle[0];
					} 
					else 
					{
						if (mostTL.X > boundingRectangle[i].X) 
						{
							mostTL.X = boundingRectangle[i].X;
						}			
						if (mostTL.Y > boundingRectangle[i].Y) 
						{
							mostTL.Y = boundingRectangle[i].Y;
						}
						if (mostBR.X < boundingRectangle[i].X)
						{
							mostBR.X = boundingRectangle[i].X;
						}
						if (mostBR.Y < boundingRectangle[i].Y)
						{
							mostBR.Y = boundingRectangle[i].Y;
						}
					}
				}

				if (mostTL.X < 0) 
				{
					if (null == m_sCursor)
					{
						//m_Hotspot.X = m_Hotspot.X + -1 * (mostTL.X);
						m_ShiftCursorAmount.X = -1 * mostTL.X;
					}
					//m_iTextX = m_iTextX - mostTL.X;
				}
				if (mostTL.Y < 0) 
				{
					if (null == m_sCursor)
					{
						//m_Hotspot.Y = m_Hotspot.Y + -1 * (mostTL.Y);
						m_ShiftCursorAmount.Y = -1 * mostTL.Y;
					}

					//m_iTextY = m_iTextY + -1 * mostTL.Y;
				}

				mostBR.X = mostBR.X + m_ShiftCursorAmount.X;
				mostBR.Y = mostBR.Y + m_ShiftCursorAmount.Y;
				Size = new Size(mostBR.X,mostBR.Y);
				Refresh();
				return;
			}

			protected override void OnPaint(PaintEventArgs e)
			{
//				if (false == m_fVisible)
//				{
//					return;
//				}


				//e->Graphics->DrawImage(m_Cursor->ToString(), 0,0);

				/*if (i== NULL) {
					i = Drawing::Image::FromFile("c:\\edward\\grouplab\\cursors\\red.cur");
				}
				PointF p;
				p = PointF(0.0,0.0);*/
				
				e.Graphics.TranslateTransform((float) m_ShiftCursorAmount.X , (float) m_ShiftCursorAmount.Y);
				
				e.Graphics.RotateTransform((float) -1.0 * m_DegreeRotation);

				

				/*e->Graphics->DrawImage(i, p);*/
				if (null == m_sCursor) 
				{
						Icon icon = Icon.FromHandle(m_Cursor.Handle);
						//icon->Size = m_Cursor->Size;
						//e.Graphics.DrawIcon(icon, m_Hotspot.X, m_Hotspot.Y);
						Bitmap b = icon.ToBitmap();
						for (int i=0; i< b.Size.Width; i++)
						{
							for (int j=0; j< b.Size.Height; j++)
							{
								Color c = b.GetPixel(i,j);
								if (255 == c.A && 242 == c.R && 244 == c.G && 243 == c.B)
								{ //if it is a inverse colour character make it black instead
									b.SetPixel(i,j,Color.Black);
								}
							}
						e.Graphics.DrawImage(b, new Rectangle(new Point(0,0),m_Cursor.Size));
					}
				
				} 
				else 
				{
					m_Cursor.Draw(e.Graphics , this.Bounds);
				}

				if (null != m_Text) 
				{
					e.Graphics.DrawString(m_Text, m_Font, m_Brush, new PointF((float) m_TextLocation.X, (float) m_TextLocation.Y), m_TextFormat);

				}


			}

			public Cursor myCursor
			{
				get
				{
					return m_Cursor;
				}
				set
				{
					Win32.ICONINFO iconinfo = new Win32.ICONINFO();
					

					bool fResult = Win32.GetIconInfo(value.Handle, ref iconinfo);
					this.m_Hotspot = new Point(iconinfo.xHotspot, iconinfo.yHotspot);
					
					//Console.WriteLine("X " + iconinfo.xHotspot + " Y " + iconinfo.yHotspot);
					Debug.Assert(false != fResult, "Could not get Hotspot information from the file");
					
					
					//Icon icon = Icon.FromHandle(value.Handle);
					//Icon icon2 = Icon.FromHandle(Cursors.IBeam.Handle);

					
					
					
					//Console.WriteLine("Height " + value.Size.Height + " Width " + value.Size.Width);
					//Console.WriteLine("Height " + icon.Height + " Width " + icon.Width);

					//fResult = (2*value.Size.Height == icon.Height && value.Size.Width == icon.Width)?true:false;
					//Debug.Assert(fResult,"Unable to load this cursor","To create a cursor go File->New File->Visual C++->Cursor File");
					
					
					m_Cursor = value;

					
					DegreeRotation = DegreeRotation;

					
					

					ResizeWindow();

				}

			}
			
			public String myText
			{
				get
				{
					return m_Text;
				}
				set
				{
					m_Text = value;
					ResizeWindow();
					Refresh();
				}
			}

			public CardinalDirections myTextCardinalPosition
			{
				get
				{
					return m_TextCardinal;
				}
				set
				{
					m_TextCardinal = value;
					StringFormat sf = new StringFormat();
					Point p = new Point(m_Cursor.Size.Width / 2, m_Cursor.Size.Height / 2);
					switch (value)
					{

						case CardinalDirections.North:
							m_TextLocation = new Point(p.X, 0);
							sf.LineAlignment = StringAlignment.Far; 
							sf.Alignment = StringAlignment.Center;
							break;
						case CardinalDirections.NorthEast:
							m_TextLocation = new Point(m_Cursor.Size.Width, 0);
							sf.LineAlignment = StringAlignment.Far;
							sf.Alignment = StringAlignment.Near;							
							break;
						case CardinalDirections.East:
							m_TextLocation = new Point(m_Cursor.Size.Width, p.Y);
							sf.LineAlignment = StringAlignment.Center;
							sf.Alignment = StringAlignment.Near;
							break;
						case CardinalDirections.SouthEast:
							m_TextLocation = new Point(m_Cursor.Size.Width, m_Cursor.Size.Height);
							sf.LineAlignment = StringAlignment.Near;
							sf.Alignment = StringAlignment.Near;
							break;
						case CardinalDirections.South:
							m_TextLocation = new Point(p.X, m_Cursor.Size.Height);
							sf.LineAlignment = StringAlignment.Near;
							sf.Alignment = StringAlignment.Center;
							break;
						case CardinalDirections.SouthWest:
							m_TextLocation = new Point(0, m_Cursor.Size.Height);
							sf.FormatFlags = StringFormatFlags.DirectionRightToLeft;
							sf.LineAlignment = StringAlignment.Near;
							sf.Alignment = StringAlignment.Near;
							break;
						case CardinalDirections.West:
							m_TextLocation = new Point(0, p.Y);
							sf.FormatFlags = StringFormatFlags.DirectionRightToLeft;
							sf.LineAlignment = StringAlignment.Center;
							sf.Alignment = StringAlignment.Near;
							break;
						case CardinalDirections.NorthWest:
							m_TextLocation = new Point(0, 0);
							sf.FormatFlags = StringFormatFlags.DirectionRightToLeft;
							sf.LineAlignment = StringAlignment.Far;
							sf.Alignment = StringAlignment.Near;
							break;
						case CardinalDirections.Normal:
							m_TextLocation = new Point(p.X, p.Y);
							sf.LineAlignment = StringAlignment.Center;
							sf.Alignment = StringAlignment.Center;
							break;						
					}
					m_TextFormat = sf;
					ResizeWindow();
					Refresh();
				}
			}

			public int RotateCursorImageOnly
			{
				get
				{
					return DegreeRotation;
				}
				set
				{
					DegreeRotation = value;
					m_RotateDelta = false;
					ResizeWindow();
					Refresh();
				}
			}


			public int DegreeRotation
			{
				get 
				{
					return m_DegreeRotation;
				}
				set
				{
					if (null == m_sCursor)
					{
						if (0 == value) 
						{
							value = 360;
						}
						value = value * -1;
						m_DegreeRotation = value;
						m_RotateDelta = true;
						double pi = 3.1415926535;
						double newval = (double) (value) * pi;
						newval = newval / 180.0;
						m_dRotationMatrix[0] = Math.Cos(newval);

						m_dRotationMatrix[1] = -1.0 * Math.Sin(newval);

						m_dRotationMatrix[2] = Math.Sin(newval);

						m_dRotationMatrix[3] = Math.Cos(newval);

						ResizeWindow();
						Refresh();
					} 
					else 
					{
						Debug.Assert(null == m_sCursor, "Can't rotate a non system cursor");
					}
				}

			}
		}
		internal MouseImpl impl;

		/// <summary>A class which inherits from ReadOnlyCollection Base and represents a collection of mice</summary>
		/// <remarks><para>To access different mice on the system a collection class was needed.  This class will 
		/// provide array access to the class and will provide a count of the total number of mice on the system.</para></remarks>
		[Description("A class which inherits from ReadOnlyCollection Base and represents a collection of mice")]
		public class MouseCollection : ReadOnlyCollectionBase
		{
			/// <summary>The default constructor for the MouseCollection class requires an array list of mice class</summary>
			public MouseCollection(ArrayList value)
			{
				foreach (Object o in value)
				{
					InnerList.Add(o);
				}
				 
				
			}

			/// <summary>Gets or sets the mouse class specified by the index parameter</summary>
			/// <remarks><para>There are several ways to iterate through all of the mice on the system.
			/// One can use a numeric iterator such as "int i" or use a mouse iterator using the foreach parameter.</para></remarks>
			/// <example><para>C# Eg., for (int i=0; i &lt; sdgManager1.Mice.Count; ++i) { sdgManager1.Mice[i].Text = "Tricks are for kids"; }</para></example>
			/// <example><para>C# Eg., foreach (Sdgt.Mouse m in sdgManager1.Mice) { m.Text = "Tricks are for kids"; }</para></example>
			[Description("Gets or sets the mouse class specified by the index parameter")]
			public Mouse this[int index]
			{
				get
				{
					return (Mouse) InnerList[index];
				}
				set
				{
					if (index == InnerList.Count)
					{
						InnerList.Add((object) value);
					}
					else
					{
						InnerList[index] = (object) value;
					}
				}
			}

			
		}
		#region Sdg Mouse Class Events

		/// <summary>Event fired when any mouse event occurs</summary>
		[Description("Event fired when any mouse event occurs")]
		public event SdgMouseEventHandler MouseMove;

		/// <summary>Event fired when any mouse button has been pressed</summary>
		[Description("Event fired when any mouse button has been pressed")]
		public event SdgMouseEventHandler MouseDown;

		/// <summary>Event fired when any mouse button has been released</summary>
		[Description("Event fired when any mouse button has been released")]
		public event SdgMouseEventHandler MouseUp;

		/// <summary>Event fired when the mouse wheel has been moved</summary>
		[Description("Event fired when the mouse wheel has been moved")]
		public event SdgMouseEventHandler MouseWheel;

		internal void OnMouseMove(SdgMouseEventArgs e)
		{
			if (MouseMove != null) 
				MouseMove(this, e);
		}
		
		internal void OnMouseDown(SdgMouseEventArgs e)
		{
			if (MouseDown != null)
				MouseDown(this, e);

		}
		
		internal void OnMouseUp(SdgMouseEventArgs e)
		{
			if (MouseUp != null)
				MouseUp(this, e);
		}
		
		internal void OnMouseWheel(SdgMouseEventArgs e)
		{
			if (MouseWheel != null)
				MouseWheel(this, e);
		}
		
		#endregion 

	}


	#region Sdg Mouse Event Code

	/// <summary>The default handler for all sdg mouse events</summary>
	[Description("The default handler for all sdg mouse events")]
	public delegate void SdgMouseEventHandler(object sender, SdgMouseEventArgs e);

	/// <summary>The event class for all sdg mouse events</summary>
	[Description("The event class for all sdg mouse events")]
	public class SdgMouseEventArgs : EventArgs
	{
		private int m_ID;
		private MouseButtons m_button;
		private int m_x;
		private int m_y;
		private int m_delta;

		/// <summary>This constructor is used for a numeric button assignment corresponding to a Raw Input Command</summary>
		public SdgMouseEventArgs(int ID, int Button, int X, int Y, int delta)
		{
			m_ID = ID;
			m_button = ConvertButton(Button);
			m_x = X;
			m_y = Y;
			if (Button != (int) Win32.RI_MOUSE.RI_MOUSE_WHEEL) 
			{
				m_delta = 0;
			} 
			else 
			{
				m_delta = delta;
			}			
		}

		/// <summary>This constructor is used for a numeric button assignment corresponding to a Raw Input Command</summary>
		public SdgMouseEventArgs(int ID, MouseButtons Button, int X, int Y, int delta)
		{
			m_ID = ID;
			m_button = Button;
			m_x = X;
			m_y = Y;
			m_delta = delta;
			
		}

		/// <summary>This constructor is used for a MouseButtons enumeration assignment</summary>
		public SdgMouseEventArgs(int ID, MouseButtons Button, int ButtonFlags, int X, int Y, int delta)
		{
			m_ID = ID;
			m_button = Button; 
			m_x = X;
			m_y = Y;
			m_delta = delta;
				
		}

		private MouseButtons ConvertButton(int Button)
			//Converts the Raw Input Data into a button value
		{
			switch ((Win32.RI_MOUSE) Button)
			{
				case Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_DOWN:
				case Win32.RI_MOUSE.RI_MOUSE_LEFT_BUTTON_UP:
					return MouseButtons.Left;
				case Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_DOWN:
				case Win32.RI_MOUSE.RI_MOUSE_MIDDLE_BUTTON_UP:
					return MouseButtons.Middle;				
				case Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_DOWN:
				case Win32.RI_MOUSE.RI_MOUSE_RIGHT_BUTTON_UP:
					return MouseButtons.Right;
				case Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_UP:
				case Win32.RI_MOUSE.RI_MOUSE_BUTTON_4_DOWN:
					return MouseButtons.XButton1;
				case Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_UP:
				case Win32.RI_MOUSE.RI_MOUSE_BUTTON_5_DOWN:
					return MouseButtons.XButton2;
				default:
					return MouseButtons.None;		
			}
		}

		/// <summary>Gets the ID of the mouse that raised this mouse event</summary>
		/// <remarks><para>This is the only property that is additional to the default windows .NET
		/// mouse event framework.  It represents the ID of the mouse from which this event originated.</para></remarks>
		/// <example><para>C# Eg., sdgManager1.Mice[e.ID].Text = "X " + e.X + " Y " + e.Y;</para></example>
		[Description("Gets the ID of the mouse that raised this mouse event")]
		public int ID
		{
			get
			{
				return m_ID;
			}
		}
		
		/// <summary>Gets the mouse buttons depressed in this mouse event</summary>
		/// <remarks><para>Please note that the mouse move event returns all mouse buttons that are 
		/// current pressed.  The mouse down and mouse up events only provide the mouse button which 
		/// raised that event.  This coincides with the .NET mouse event standards.  For the mouse
		/// move events it is advisable to use the &amp; operator to determine if a particular mouse button
		/// has been pressed.</para></remarks>
		/// <example><para>C# Eg., if ((e.Button &amp; MouseButtons.Left) &gt; 0) {}</para></example>
		/// <example><para>C# Eg., if (e.Button == MouseButtons.Left) {}</para></example>
		[Description("Gets the mouse buttons depressed in this mouse event")]
		public MouseButtons Button
		{
			get
			{
				return m_button;
			}
		}

		/// <summary>Gets the X position of this mouse event</summary>
		/// <remarks><para>Please note that the X position provided by this event parameter is always
		/// relative to the Control specified in the RelativeTo property.  If the ScaleMode property is
		/// set to "PixelDelta" then all of the coordinates are relative to the last point provided. </para></remarks>
		/// <example><para>C# Eg., g.DrawLine(Pens.Black, new Point(e.X,e.Y), new Point(e.X+2,e.Y+2));</para></example>
		[Description("Gets the X position of this mouse event")]
		public int X
		{
			get
			{
				return m_x;
			}
			set
			{
				m_x = value;
			}
		}

		/// <summary>Gets the Y position of this mouse event</summary>
		/// <remarks><para>Please note that the Y position provided by this event parameter is always
		/// relative to the Control specified in the RelativeTo property.  If the ScaleMode property is
		/// set to "PixelDelta" then all of the coordinates are relative to the last point provided. </para></remarks>
		/// <example><para>C# Eg., g.DrawLine(Pens.Black, new Point(e.X,e.Y), new Point(e.X+2,e.Y+2));</para></example>
		[Description("Gets the Y position of this mouse event")]
		public int Y
		{
			get
			{
				return m_y;
			}
			set
			{
				m_y = value;
			}
		}

		/// <summary>Gets the number of detantes the mouse wheel has been moved multiplied by 120</summary>
		/// <remarks><para>A shift of the scroll wheel is called a detante. Please note that windows 
		/// by default returns the number of detantes moved multiplied by 120.  This means that a 
		/// Delta value of 120 means that the mouse moveed up one Detante.  -240 means the mouse moved down
		/// two detantes.</para></remarks>
		/// <example><para>C# Eg., int iDetantesMoved = e.Delta / 120; </para></example>
		[Description("Gets the number of detantes the mouse wheel has been moved in this mouse event")]
		public int Delta
		{
			get
			{
				return m_delta;
			}
		}


	}

	#endregion
}