 using System;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using System.Windows.Forms;
using System.Drawing;

namespace Sdgt
{
	/// <summary>This is the required interface for all sdg Mouse Widgets, it provides all of the required functionality that any sdg mouse widget would need</summary>
	[Description("This is the required interface for all sdg Mouse Widgets, it provides all of the required functionality that any sdg mouse widget would need")]
	public interface ISdgMouseWidget
	{
		/// <summary>Method fired whenever a mouse is moved over this widget</summary>
		[Description("Method fired whenever a mouse is moved over this widget")]
		void OnSdgMouseMove(SdgMouseEventArgs e);

		/// <summary>Method fired whenever a mouse button is pressed over this widget</summary>
		[Description("Method fired whenever a mouse button is pressed over this widget")]
		void OnSdgMouseDown(SdgMouseEventArgs e);

		/// <summary>Method fired whenever a mouse button is released after pressing the widget</summary>
		[Description("Method fired whenever a mouse button is released after pressing the widget")]
		void OnSdgMouseUp(SdgMouseEventArgs e);

		/// <summary>Method fired whenever a mouse button is released over this widget</summary>
//		[Description("Method fired whenever a mouse button is released over this widget")]
		void OnSdgMouseClick(SdgMouseEventArgs e);
		//Add Drag, Hover, Enter, Leave functionality (later)

	}

	/// <summary>This is the required interface for sdg Keyboard Widgets, it provides all of the required functionality that any sdg keyboard enabled widget would need</summary>
	[Description("This is the required interface for sdg Keyboard Widgets, it provides all of the required functionality that any sdg keyboard enabled widget would need")]
	public interface ISdgKeyboardWidget : ISdgMouseWidget
	{
		/// <summary>Method fired whenever a keyboard event is pressed and the current widget is focused by that mouse</summary>
		[Description("Method fired whenever a keyboard event is pressed and the current widget is focused by that mouse")]
		void OnSdgKeyPress(SdgKeyPressEventArgs e);

		/// <summary>Method fired whenever a keyboard event is pressed and the current widget is focused by that mouse</summary>
		[Description("Method fired whenever a keyboard event is pressed and the current widget is focused by that mouse")]
		void OnSdgKeyDown(SdgKeyEventArgs e);

		/// <summary>Method fired whenever a keyboard event is released and the current widget is focused by that mouse</summary>
		[Description("Method fired whenever a keyboard event is released and the current widget is focused by that mouse")]
		void OnSdgKeyUp(SdgKeyEventArgs e);
	}

	/// <summary>A Base sdg Control Class.  This provides all of the base sdgWidget Functionality</summary>
	[Description("A Base sdg Control Class.  This provides all of the base sdgWidget Functionality"),
	DefaultEvent("SdgMouseDown")]
	public class SdgControl : Control, ISdgKeyboardWidget 
	{
		
		/// <summary>The Default Sdg Control constructor</summary>
		public SdgControl()
		{
			this.AllowDrop = false;
			this.Capture = false;
			this.CausesValidation = false;
			
			
		}

		
		private object DetermineSender(Point p)
		{
			
			for (int i=0; i< this.Controls.Count; i++)
			{

				Control c = this.Controls[i];
				if (c.Bounds.Contains(p))
				{
					return (object) this.Controls[i];
				}
			}
			return this;
		}

		#region Events

		/// <summary>Method fired whenever a mouse is moved over this widget</summary>
		[Description("Method fired whenever a mouse is moved over this widget")]
		public void OnSdgMouseMove(SdgMouseEventArgs e)
		{
			if (null != SdgMouseMove)
				SdgMouseMove(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is pressed over this widget</summary>
		[Description("Method fired whenever a mouse button is pressed over this widget")]
		public void OnSdgMouseDown(SdgMouseEventArgs e)
		{	
			if (null != SdgMouseDown)
				SdgMouseDown(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released after pressing on this widget</summary>
		[Description("Method fired whenever a mouse button is released after pressing on this widget")]
		public void OnSdgMouseUp(SdgMouseEventArgs e)
		{
			if (null != SdgMouseUp)
				SdgMouseUp(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released over this widget</summary>
		[Description("Method fired whenever a mouse button is released over this widget")]
		public void OnSdgMouseClick(SdgMouseEventArgs e)
		{
			if (null != SdgMouseClick)
				SdgMouseClick(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyPress(SdgKeyPressEventArgs e)
		{
			if (null != SdgKeyPress)
				SdgKeyPress(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyDown(SdgKeyEventArgs e)
		{
			if (null != SdgKeyDown)
				SdgKeyDown(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]
		public void OnSdgKeyUp(SdgKeyEventArgs e)
		{
			if (null != SdgKeyUp)
				SdgKeyUp(this, e);
		}

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]		
		public event SdgKeyEventHandler SdgKeyUp;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyEventHandler SdgKeyDown;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyPressEventHandler SdgKeyPress;

		/// <summary>Event fired when any mouse button has been released over this widget</summary>
		[Description("Event fired when any mouse button has been released over this widget")]
		public event SdgMouseEventHandler SdgMouseClick;

		/// <summary>Event fired when any mouse event occurs on this widget</summary>
		[Description("Event fired when any mouse event occurs")]
		public event SdgMouseEventHandler SdgMouseMove;

		/// <summary>Event fired when any mouse button has been pressed on this widget</summary>
		[Description("Event fired when any mouse button has been pressed")]
		public event SdgMouseEventHandler SdgMouseDown;

		/// <summary>Event fired when any mouse button has been released after pressing on this widget</summary>
		[Description("Event fired when any mouse button has been released after pressing on this widget")]
		public event SdgMouseEventHandler SdgMouseUp;

		#endregion

	}

	/// <summary>A Base sdg User Control Class.  This provides all of the base sdgWidget Functionality</summary>
	[Description("A Base sdg User Control Class.  This provides all of the base sdgWidget Functionality"), DefaultEvent("SdgMouseDown")]
	public class SdgUserControl : UserControl, ISdgKeyboardWidget
	{
		/// <summary>The Default Sdg Control constructor</summary>
		public SdgUserControl()
		{
			this.AllowDrop = false;
			this.Capture = false;
			this.CausesValidation = false;
			
		}
		private object DetermineSender(Point p)
		{
			
			for (int i=0; i< this.Controls.Count; i++)
			{

				Control c = this.Controls[i];
				//Console.WriteLine("The Control is " + c.Text);
				if (c.Bounds.Contains(p))
				{
					return (object) this.Controls[i];
				}
			}
			return this;
		}

		#region Events

		/// <summary>Method fired whenever a mouse is moved over this widget</summary>
		[Description("Method fired whenever a mouse is moved over this widget")]
		public void OnSdgMouseMove(SdgMouseEventArgs e)
		{
			if (null != SdgMouseMove)
				SdgMouseMove(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is pressed over this widget</summary>
		[Description("Method fired whenever a mouse button is pressed over this widget")]
		public void OnSdgMouseDown(SdgMouseEventArgs e)
		{	
			if (null != SdgMouseDown)
				SdgMouseDown(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released after pressing on this widget</summary>
		[Description("Method fired whenever a mouse button is released after pressing on this widget")]
		public void OnSdgMouseUp(SdgMouseEventArgs e)
		{
			if (null != SdgMouseUp)
				SdgMouseUp(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released over this widget</summary>
		[Description("Method fired whenever a mouse button is released over this widget")]
		public void OnSdgMouseClick(SdgMouseEventArgs e)
		{
			if (null != SdgMouseClick)
				SdgMouseClick(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyPress(SdgKeyPressEventArgs e)
		{
			if (null != SdgKeyPress)
				SdgKeyPress(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyDown(SdgKeyEventArgs e)
		{
			if (null != SdgKeyDown)
				SdgKeyDown(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]
		public void OnSdgKeyUp(SdgKeyEventArgs e)
		{
			if (null != SdgKeyUp)
				SdgKeyUp(this, e);
		}

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]		
		public event SdgKeyEventHandler SdgKeyUp;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyEventHandler SdgKeyDown;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyPressEventHandler SdgKeyPress;

		/// <summary>Event fired when any mouse button has been released over this widget</summary>
		[Description("Event fired when any mouse button has been released over this widget")]
		public event SdgMouseEventHandler SdgMouseClick;

		/// <summary>Event fired when any mouse event occurs on this widget</summary>
		[Description("Event fired when any mouse event occurs")]
		public event SdgMouseEventHandler SdgMouseMove;

		/// <summary>Event fired when any mouse button has been pressed on this widget</summary>
		[Description("Event fired when any mouse button has been pressed")]
		public event SdgMouseEventHandler SdgMouseDown;

		/// <summary>Event fired when any mouse button has been released after pressing on this widget</summary>
		[Description("Event fired when any mouse button has been released after pressing on this widget")]
		public event SdgMouseEventHandler SdgMouseUp;

		#endregion

	}

	/// <summary>A Base Sdg Form Class.  Any active form must be registered with the SDG Manager in order to recieve events</summary>
	[Description("A Base Sdg Form Class.  Any active form must be registered with the SDG Manager in order to recieve events"), DefaultEvent("SdgMouseDown")]
	public class SdgForm : Form, ISdgKeyboardWidget
	{
		/// <summary>The Default Sdg Form Constructor</summary>
		public SdgForm()
		{
			this.FormBorderStyle = FormBorderStyle.None;
			//this.TransparencyKey = this.BackColor;
			this.TopMost = true;
			this.Opacity = 50;
			this.Size = new Size(200,200);
			
		}
		private object DetermineSender(Point p)
		{
			
			for (int i=0; i< this.Controls.Count; i++)
			{

				Control c = this.Controls[i];
				//Console.WriteLine("The Control is " + c.Text);
				if (c.Bounds.Contains(p))
				{
					return (object) this.Controls[i];
				}
			}
			return this;
		}

		#region Events

		/// <summary>Method fired whenever a mouse is moved over this widget</summary>
		[Description("Method fired whenever a mouse is moved over this widget")]
		public void OnSdgMouseMove(SdgMouseEventArgs e)
		{
			if (null != SdgMouseMove)
				SdgMouseMove(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is pressed over this widget</summary>
		[Description("Method fired whenever a mouse button is pressed over this widget")]
		public void OnSdgMouseDown(SdgMouseEventArgs e)
		{	
			if (null != SdgMouseDown)
				SdgMouseDown(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released after pressing on this widget</summary>
		[Description("Method fired whenever a mouse button is released after pressing on this widget")]
		public void OnSdgMouseUp(SdgMouseEventArgs e)
		{
			if (null != SdgMouseUp)
				SdgMouseUp(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever a mouse button is released over this widget</summary>
		[Description("Method fired whenever a mouse button is released over this widget")]
		public void OnSdgMouseClick(SdgMouseEventArgs e)
		{
			if (null != SdgMouseClick)
				SdgMouseClick(DetermineSender(new Point(e.X, e.Y)), e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyPress(SdgKeyPressEventArgs e)
		{
			if (null != SdgKeyPress)
				SdgKeyPress(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]
		public void OnSdgKeyDown(SdgKeyEventArgs e)
		{
			if (null != SdgKeyDown)
				SdgKeyDown(this, e);
		}

		/// <summary>Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Method fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]
		public void OnSdgKeyUp(SdgKeyEventArgs e)
		{
			if (null != SdgKeyUp)
				SdgKeyUp(this, e);
		}

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is released")]		
		public event SdgKeyEventHandler SdgKeyUp;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyEventHandler SdgKeyDown;

		/// <summary>Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed</summary>
		[Description("Event fired whenever this widget has mouse focus and the a keyboard mapped to that mouse is pressed")]		
		public event SdgKeyPressEventHandler SdgKeyPress;

		/// <summary>Event fired when any mouse button has been released over this widget</summary>
		[Description("Event fired when any mouse button has been released over this widget")]
		public event SdgMouseEventHandler SdgMouseClick;

		/// <summary>Event fired when any mouse event occurs on this widget</summary>
		[Description("Event fired when any mouse event occurs")]
		public event SdgMouseEventHandler SdgMouseMove;

		/// <summary>Event fired when any mouse button has been pressed on this widget</summary>
		[Description("Event fired when any mouse button has been pressed")]
		public event SdgMouseEventHandler SdgMouseDown;

		/// <summary>Event fired when any mouse button has been released after pressing on this widget</summary>
		[Description("Event fired when any mouse button has been released after pressing on this widget")]
		public event SdgMouseEventHandler SdgMouseUp;

		#endregion

		
	}

	
}

